@@ -19,6 +19,8 @@
 
 package edu.berkeley.boinc;
 
+import edu.berkeley.boinc.utils.*;
+
 import java.net.URL;
 import edu.berkeley.boinc.client.Monitor;
 import android.app.Activity;
@@ -33,29 +35,27 @@ import android.os.AsyncTask;
 import android.os.Bundle;
 import android.os.IBinder;
 import android.text.InputType;
+import android.text.SpannableString;
+import android.text.style.UnderlineSpan;
 import android.util.Log;
 import android.view.View;
+import android.view.Window;
 import android.widget.Button;
 import android.widget.EditText;
 import android.widget.ImageView;
 import android.widget.LinearLayout;
-import android.widget.ProgressBar;
 import android.widget.TextView;
 import android.widget.Toast;
-import edu.berkeley.boinc.rpc.AccountOut;
 import edu.berkeley.boinc.rpc.PlatformInfo;
 import edu.berkeley.boinc.rpc.ProjectConfig;
 import edu.berkeley.boinc.rpc.ProjectInfo;
-import edu.berkeley.boinc.utils.BOINCErrors;
 
 public class AttachProjectLoginActivity extends Activity{
 	
-	private final String TAG = "BOINC AttachProjectLoginActivity"; 
-	
 	private Monitor monitor;
-	private Boolean mIsBound;
+	private Boolean mIsBound = false;
 	
-	private String url;
+	private String url = "";
 	private ProjectInfo projectInfo;
 	private Boolean projectInfoPresent = false; // complete ProjectInfo available, if selection from list
 	private ProjectConfig projectConfig;
@@ -67,7 +67,7 @@ public class AttachProjectLoginActivity extends Activity{
 	        monitor = ((Monitor.LocalBinder)service).getService();
 		    mIsBound = true;
 		    
-		    (new GetProjectConfig()).execute((Void[]) null);
+		    (new GetProjectConfig()).execute(url);
 	    }
 
 	    public void onServiceDisconnected(ComponentName className) { // This should not happen
@@ -79,50 +79,68 @@ public class AttachProjectLoginActivity extends Activity{
     @Override
     public void onCreate(Bundle savedInstanceState) {  
         super.onCreate(savedInstanceState);  
-        Log.d(TAG, "onCreate"); 
+        requestWindowFeature(Window.FEATURE_CUSTOM_TITLE);
 
     	//parse master url from intent extras
         Boolean urlPresent = false;
         try {
         	url = getIntent().getCharSequenceExtra("url").toString();
-        	Log.d(TAG,"url: " + url);
+        	if(Logging.DEBUG) Log.d(Logging.TAG,"AttachProjectLoginActivity onCreate with url: " + url);
         	if(url != null) urlPresent = true;
         } catch (Exception e) {}
         
         //parse  project info from intent extras
         try {
         	projectInfo = (ProjectInfo) getIntent().getSerializableExtra("projectInfo");
-        	Log.d(TAG,"projectInfo: " + projectInfo);
+        	if(Logging.VERBOSE) Log.v(Logging.TAG,"projectInfo: " + projectInfo);
         	if(projectInfo != null) {
         		projectInfoPresent = true;
         		url = projectInfo.url; // set url field to information of projectInfo
         	}
-        } catch (Exception e) {Log.d(TAG,"no project info...");}
+        } catch (Exception e) {if(Logging.DEBUG) Log.d(Logging.TAG,"no project info...");}
         
         if(!projectInfoPresent) { // url can not be taken of ProjectInfo
         	// format user input on URL right to avoid exceptions
         	if (!url.startsWith("http://") && !url.startsWith("https://")) url = "http://" + url; // add http:// in case user leaves it out
         	if (!url.endsWith("/")) url = url + "/"; // add trailing slash
+        	if(Logging.DEBUG) Log.d(Logging.TAG,"onCreate url sanitized to: " + url);
         }
         
         if(!urlPresent && !projectInfoPresent) {
         	// neither url (manual input) nor project info (list selection) is present
-        	Log.d(TAG,"neither url nor projectInfo available! finish activity...");
+        	if(Logging.WARNING) Log.w(Logging.TAG,"neither url nor projectInfo available! finish activity...");
         	finish(R.string.attachproject_login_error_toast);
         }
         
         // setup layout
-        setContentView(R.layout.attach_project_login_layout_loading);
-        TextView urlView = (TextView) findViewById(R.id.login_loading_url);
-      	urlView.setText(url);
+        setContentView(R.layout.generic_layout_loading);
+        TextView header = (TextView) findViewById(R.id.loading_header);
+        header.setText(R.string.attachproject_login_loading);
+        TextView subHeader = (TextView) findViewById(R.id.loading_subheader);
+        subHeader.setVisibility(View.VISIBLE);
+        subHeader.setText(url);
+        
+        // set title bar
+        getWindow().setFeatureInt(Window.FEATURE_CUSTOM_TITLE, R.layout.title_bar);
         
         // bind monitor service
         doBindService();
     }
     
+	@Override
+	protected void onResume() {
+		// clear the password input from edittext
+		// scenario: login fails and user navigates back to this activity
+		try {
+			EditText pwdInput = (EditText) findViewById(R.id.pwd_input);
+			pwdInput.setText("");
+		} catch (Exception e){} // catch exception because edittext does not exist upon first onResume.
+		super.onResume();
+	}
+
 	@Override
 	protected void onDestroy() {
-    	Log.d(TAG, "onDestroy");
+    	if(Logging.VERBOSE) Log.v(Logging.TAG, "AttachProjectLoginActivity onDestroy");
 	    doUnbindService();
 	    super.onDestroy();
 	}
@@ -147,22 +165,20 @@ public class AttachProjectLoginActivity extends Activity{
 	}
 	
 	// gets called by GetProjectConfig when ProjectConfig is available
-	private void populateLayout() {
-		Log.d(TAG, "populateLayout");
+	private void populateLayout(Boolean projectAlreadyAttached) {
+		if(Logging.DEBUG) Log.d(Logging.TAG, "AttachProjectLoginActivity.populateLayout: projectAlreadyAttached: " + projectAlreadyAttached);
 		
 		setContentView(R.layout.attach_project_login_layout);
 		
 		// set name
 		TextView name = (TextView) findViewById(R.id.project_name);
-		if(projectLogo != null) {
-			name.setVisibility(View.GONE); // disable name if logo present
-		} else {
-			name.setText(projectConfig.name);
-		}
+		name.setText(projectConfig.name);
 		
 		// set website
 		TextView website = (TextView) findViewById(R.id.project_url);
-		website.setText(projectConfig.masterUrl);
+    	SpannableString content = new SpannableString(projectConfig.masterUrl);
+    	content.setSpan(new UnderlineSpan(), 0, content.length(), 0);
+		website.setText(content);
 		website.setTag(projectConfig.masterUrl); // set tag to use in onClick
 		
 		// set android support
@@ -171,7 +187,8 @@ public class AttachProjectLoginActivity extends Activity{
 			platform.setText(R.string.attachproject_login_platform_supported);
 		} else {
 			platform.setText(R.string.attachproject_login_platform_not_supported);
-			platform.setTextColor(getResources().getColor(R.color.warning));
+			platform.setBackgroundResource(R.drawable.shape_yellow_background);
+			platform.setPadding(3, 3, 3, 3);
 		}
 		
 		// set ProjectInfo fields, if ProjectInfo available (after list selection)
@@ -236,125 +253,122 @@ public class AttachProjectLoginActivity extends Activity{
 		}
 		
 		// set account creation
-		TextView creationCategory = (TextView) findViewById(R.id.category_creation);
-		creationCategory.setText(getString(R.string.attachproject_login_category_creation) + " " + projectConfig.name + "?");
-		TextView creationText = (TextView) findViewById(R.id.creation_action);
-		Button creationSubmit = (Button) findViewById(R.id.registration_button);
-		if(projectConfig.accountCreationDisabled) {
-			creationText.setText(R.string.attachproject_login_header_creation_disabled);
-			creationSubmit.setVisibility(View.GONE);
-		} else if(projectConfig.clientAccountCreationDisabled) {
-			creationText.setText(R.string.attachproject_login_header_creation_client_disabled);
-			creationSubmit.setTag(false);
-		} else {
-			// creation in client supported
-			creationText.setText(R.string.attachproject_login_header_creation_enabled);
-			creationSubmit.setTag(true);
+		if(projectAlreadyAttached) {
+			LinearLayout creationWrapper = (LinearLayout) findViewById(R.id.creation_wrapper);
+			creationWrapper.setVisibility(View.GONE);
+		} else{
+			TextView creationCategory = (TextView) findViewById(R.id.category_creation);
+			creationCategory.setText(getString(R.string.attachproject_login_category_creation) + " " + projectConfig.name + "?");
+			TextView creationText = (TextView) findViewById(R.id.creation_action);
+			Button creationSubmit = (Button) findViewById(R.id.registration_button);
+			if(projectConfig.accountCreationDisabled) {
+				creationText.setText(R.string.attachproject_login_header_creation_disabled);
+				creationSubmit.setVisibility(View.GONE);
+			} else if(projectConfig.clientAccountCreationDisabled) {
+				creationText.setText(R.string.attachproject_login_header_creation_client_disabled);
+				creationSubmit.setTag(false);
+			} else {
+				// creation in client supported
+				creationText.setText(R.string.attachproject_login_header_creation_enabled);
+				creationSubmit.setTag(true);
+			}
 		}
 		
 		// set account login
-		TextView loginCategory = (TextView) findViewById(R.id.category_login);
-		loginCategory.setText(R.string.attachproject_login_category_login);
-		if(projectConfig.userName) { // user vs. email?
-			TextView idHeader = (TextView) findViewById(R.id.header_id);
-			idHeader.setText(R.string.attachproject_login_header_id_name);
-			EditText idInput = (EditText) findViewById(R.id.id_input);
-			idInput.setInputType(InputType.TYPE_TEXT_FLAG_NO_SUGGESTIONS);
+		if(projectAlreadyAttached) {
+			LinearLayout loginWrapper = (LinearLayout) findViewById(R.id.login_wrapper);
+			loginWrapper.setVisibility(View.GONE);
+		} else {
+			TextView loginCategory = (TextView) findViewById(R.id.category_login);
+			loginCategory.setText(R.string.attachproject_login_category_login);
+			if(projectConfig.userName) { // user vs. email?
+				if(Logging.DEBUG) Log.d(Logging.TAG,"project is using user name instead of email for login");
+				TextView idHeader = (TextView) findViewById(R.id.header_id);
+				idHeader.setText(R.string.attachproject_login_header_id_name);
+				EditText idInput = (EditText) findViewById(R.id.id_input);
+				idInput.setInputType(InputType.TYPE_TEXT_FLAG_NO_SUGGESTIONS);
+			}
+		}
+		
+		// set project attached
+		if(projectAlreadyAttached) {
+			LinearLayout attachedWrapper = (LinearLayout) findViewById(R.id.attached_wrapper);
+			attachedWrapper.setVisibility(View.VISIBLE);
+			TextView header = (TextView) attachedWrapper.findViewById(R.id.header);
+			header.setText(R.string.attachproject_login_attached);
 		}
 	}
 	
 	public void login (View view) {
-		new ProjectLoginAsync().execute();
+		// parse user input
+		EditText idInput = (EditText) findViewById(R.id.id_input);
+		EditText pwdInput = (EditText) findViewById(R.id.pwd_input);
+		String id = idInput.getText().toString();
+		String pwd = pwdInput.getText().toString();
+		
+		// verify input and start AttachProjectWorkingActivity
+		if(verifyInput(id, pwd)){
+			Intent intent = new Intent(this, AttachProjectWorkingActivity.class);
+			intent.putExtra("registration", false);
+			intent.putExtra("usesName", projectConfig.userName);
+			intent.putExtra("projectUrl", projectConfig.masterUrl);
+			intent.putExtra("projectName", projectConfig.name);
+			intent.putExtra("id", id);
+			intent.putExtra("pwd", pwd);
+			startActivity(intent);
+		}
 	}
 	
 	// register button's onClick
 	public void register (View view) {
-		Log.d(TAG, "register: " + view.getTag());
+		if(Logging.DEBUG) Log.d(Logging.TAG, "register: " + view.getTag());
 		Boolean clientCreation = (Boolean) view.getTag();
 		if (clientCreation) {
-			// start intent to AttachProjectRegistrationActivity
+			// start intent to AttachProjectWorkingActivity
 			Intent intent = new Intent(this, AttachProjectRegistrationActivity.class);
-			intent.putExtra("projectUrl", url);
+			intent.putExtra("projectUrl", projectConfig.masterUrl);
 			intent.putExtra("projectName", projectConfig.name);
 			intent.putExtra("minPwdLength", projectConfig.minPwdLength);
+			intent.putExtra("usesName", projectConfig.userName);
 			startActivity(intent);
 		} else {
 			// start intent to project website
 			Intent i = new Intent(Intent.ACTION_VIEW);
-			i.setData(Uri.parse(url));
+			i.setData(Uri.parse(projectConfig.masterUrl));
 			startActivity(i);
 		}
 	}
 	
+	//project forgot password button onClick
+	public void forgotPasswd (View view) {
+		// Start intent to project website forgot password link
+		Intent i = new Intent(Intent.ACTION_VIEW);
+		i.setData(Uri.parse(projectConfig.masterUrl + "/get_passwd.php"));
+		startActivity(i);
+	}
+	
 	// project url textview's onClick
 	public void projectUrlClicked (View view) {
 		// start intent to project website
 		Intent i = new Intent(Intent.ACTION_VIEW);
-		i.setData(Uri.parse(url));
+		i.setData(Uri.parse(projectConfig.masterUrl));
 		startActivity(i);
 	}
 	
 	private Boolean verifyInput(String id, String pwd) {
 		if(id.length() == 0) {
-			Toast toast = Toast.makeText(getApplicationContext(), R.string.attachproject_login_toast_error_no_name, Toast.LENGTH_SHORT);
+			Toast toast = Toast.makeText(getApplicationContext(), R.string.attachproject_error_no_name, Toast.LENGTH_SHORT);
 			toast.show();
 			return false;
 		}
 		if(pwd.length() == 0) {
-			Toast toast = Toast.makeText(getApplicationContext(), R.string.attachproject_login_toast_error_no_pwd, Toast.LENGTH_SHORT);
+			Toast toast = Toast.makeText(getApplicationContext(), R.string.attachproject_error_no_pwd, Toast.LENGTH_SHORT);
 			toast.show();
 			return false;
 		}
 		return true;
 	}
 	
-	private void showResultToast(Integer code) {
-		Log.d(TAG,"showResultToast for error: " + code);
-		Toast toast;
-		if(code == null) {
-			toast = Toast.makeText(getApplicationContext(), R.string.attachproject_login_toast_error_unknown, Toast.LENGTH_LONG);
-		} else {
-			switch (code) {
-			case BOINCErrors.ERR_OK:
-				toast = Toast.makeText(getApplicationContext(), R.string.attachproject_login_toast_ok, Toast.LENGTH_LONG);
-				break;
-			case BOINCErrors.ERR_BAD_PASSWD:
-				toast = Toast.makeText(getApplicationContext(), R.string.attachproject_login_toast_error_wrong_pwd, Toast.LENGTH_LONG);
-				break;
-			case BOINCErrors.ERR_DB_NOT_FOUND:
-				toast = Toast.makeText(getApplicationContext(), R.string.attachproject_login_toast_error_wrong_name, Toast.LENGTH_LONG);
-				break;
-			case BOINCErrors.ERR_GETHOSTBYNAME:
-				toast = Toast.makeText(getApplicationContext(), R.string.attachproject_login_toast_error_no_internet, Toast.LENGTH_LONG);
-				break;
-			default:
-				toast = Toast.makeText(getApplicationContext(), R.string.attachproject_login_toast_error_unknown, Toast.LENGTH_LONG);
-				break;
-			}
-		}
-		toast.show();
-	}
-	
-	private void changeLayoutLoggingIn(Boolean loggingIn) {
-		Log.d(TAG,"changeLayoutLoggingIn: " + loggingIn);
-		LinearLayout loginWrapper = (LinearLayout) findViewById(R.id.login_wrapper);
-		ProgressBar loginLoading = (ProgressBar) findViewById(R.id.login_pending);
-		
-		if(loggingIn) {
-			loginWrapper.setVisibility(View.GONE);
-			loginLoading.setVisibility(View.VISIBLE);
-		} else {
-			loginLoading.setVisibility(View.GONE);
-			loginWrapper.setVisibility(View.VISIBLE);
-		}
-	}
-	
-	private void goToMainActivity() {
-		Intent intent = new Intent(this, BOINCActivity.class);
-		intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP); //clear_top closes AttachProjectListActivity!
-		startActivity(intent);
-	}
-	
 	private Boolean platformSupported() {
 		if(projectConfig == null) return false;
 		String platformName = getString(R.string.boinc_platform_name);
@@ -368,100 +382,54 @@ public class AttachProjectLoginActivity extends Activity{
 		return supported;
 	}
 	
-	private final class ProjectLoginAsync extends AsyncTask<Void, Void, Integer> {
-
-		//private final String TAG = "ProjectLoginAsync";
+	private final class GetProjectConfig extends AsyncTask<String, Void, Integer> {
 		
-		private String id;
-		private String pwd;
-		
-		@Override 
-		protected void onPreExecute() {
-			changeLayoutLoggingIn(true);
-			
-			// read input data
-			EditText idInput = (EditText) findViewById(R.id.id_input);
-			EditText pwdInput = (EditText) findViewById(R.id.pwd_input);
-			id = idInput.getText().toString();
-			pwd = pwdInput.getText().toString();
-
-			if(!verifyInput(id, pwd)) {
-				changeLayoutLoggingIn(false);
-				cancel(true); // cancel asynctask if validation fails
-			}
-			
-		}
+		private boolean projectAlreadyAttached = false;
 		
 		@Override
-		protected Integer doInBackground(Void... params) {
-			
-			AccountOut account = monitor.lookupCredentials(url, id, pwd);
-			
-			try {
-				if(account.error_num == BOINCErrors.ERR_OK) {
-					Boolean attach = monitor.attachProject(url, id, account.authenticator);
-					if(attach) {
-						return BOINCErrors.ERR_OK;
+		protected Integer doInBackground(String... params) {
+			String url = params[0];
+			Integer attemptCounter = 0;
+			Integer maxAttempts = getResources().getInteger(R.integer.attach_get_project_config_retries);
+			try{
+				while(attemptCounter < maxAttempts) {
+					if(!projectInfoPresent) { // only url string is available
+						if(Logging.DEBUG) Log.d(Logging.TAG, "doInBackground() - GetProjectConfig for manual input url: " + url);
+						
+						//fetch ProjectConfig
+						projectConfig = monitor.getProjectConfig(url);
 					} else {
-						Log.d(TAG,"attachProject failed");
-						// happens if project already attached
-						return null;
+						if(Logging.DEBUG) Log.d(Logging.TAG, "doInBackground() - GetProjectConfig for list selection url: " + projectInfo.url);
+						
+						//fetch ProjectConfig
+						projectConfig = monitor.getProjectConfig(projectInfo.url);
+						
+						// fetch project logo	
+						loadBitmap();
+					}
+					
+					if (projectConfig != null && projectConfig.error_num != null && projectConfig.error_num == 0) {
+						// success
+						projectAlreadyAttached = checkProjectAlreadyAttached(url);
+						return 0;
+					} else { 
+						if(Logging.DEBUG) if(projectConfig != null) Log.d(Logging.TAG,"getProjectConfig returned error num:" + projectConfig.error_num);
+						attemptCounter++;
 					}
-				} else { // error code
-					return account.error_num;
 				}
-			} catch (Exception e) {}
-			return null;
-		}
-		
-		@Override
-		protected void onPostExecute(Integer errorCode) {
-			showResultToast(errorCode);
-			if((errorCode != null) && (errorCode == BOINCErrors.ERR_OK)) { //successful
-				monitor.forceRefresh(); // force refresh, so "no project banner" disappears
-				goToMainActivity();
-			}
-			changeLayoutLoggingIn(false);
-		}
-	}
-	
-	private final class GetProjectConfig extends AsyncTask<Void, Void, Integer> {
-
-		private final String TAG = "GetProjectConfig";
-		
-		@Override
-		protected Integer doInBackground(Void... params) {
-			if(!projectInfoPresent) { // only url string is available
-				Log.d(TAG, "doInBackground() - GetProjectConfig for manual input url: " + url);
-				
-				if(checkProjectAlreadyAttached(url)) return R.string.attachproject_login_error_project_exists;
-				
-				//fetch ProjectConfig
-				projectConfig = monitor.getProjectConfig(url);
-			} else {
-				Log.d(TAG, "doInBackground() - GetProjectConfig for list selection url: " + projectInfo.url);
-				
-				if(checkProjectAlreadyAttached(projectInfo.url)) return R.string.attachproject_login_error_project_exists;
-				
-				//fetch ProjectConfig
-				projectConfig = monitor.getProjectConfig(projectInfo.url);
-				
-				// fetch project logo	
-				loadBitmap();
-			}
-			
-			if (projectConfig != null && projectConfig.error_num != null && projectConfig.error_num == 0) {
-				return 0;
-			} else { 
-				Log.d(TAG,"getProjectConfig returned error num:" + projectConfig.error_num);
-				return R.string.attachproject_login_error_toast;
+			} catch(Exception e) {
+				if(Logging.WARNING) Log.w(Logging.TAG,"error in doInBackround",e);
 			}
+			// if this code is reached, it failed, return
+			if(Logging.DEBUG) if(projectConfig != null) Log.d(Logging.TAG,"getProjectConfig returned error num:" + projectConfig.error_num);
+			return R.string.attachproject_login_error_toast;
 		}
 		
 		@Override
 		protected void onPostExecute(Integer toastStringId) {
 			if(toastStringId == 0) { // no error, no toast...
-				populateLayout();
+				if(Logging.DEBUG) Log.d(Logging.TAG, "onPostExecute() - GetProjectConfig successful.");
+				populateLayout(projectAlreadyAttached);
 			} else {
 				finish(toastStringId);
 			}
@@ -473,14 +441,14 @@ public class AttachProjectLoginActivity extends Activity{
 			try {
 				URL logoUrlUrl = new URL(projectInfo.imageUrl);
 				projectLogo = BitmapFactory.decodeStream(logoUrlUrl.openConnection().getInputStream());
-				if(projectLogo!=null) Log.d(TAG, "logo download successful.");
+				if(projectLogo!=null) if(Logging.DEBUG) Log.d(Logging.TAG, "logo download successful.");
 			} catch (Exception e) {
-				Log.w(TAG,"loadBitmap failed.",e);
+				if(Logging.WARNING) Log.w(Logging.TAG,"loadBitmap failed.",e);
 			}
 		}
 		
 		private Boolean checkProjectAlreadyAttached(String url) {
-			Log.d(TAG, "check whether project with url is already attached: " + url);
+			if(Logging.DEBUG) Log.d(Logging.TAG, "check whether project with url is already attached: " + url);
 			return monitor.checkProjectAttached(url);
 		}
 	}
