@@ -28,14 +28,15 @@ import java.net.InetSocketAddress;
 import java.net.Socket;
 import java.net.UnknownHostException;
 import java.util.ArrayList;
+import java.util.Locale;
 
 import org.xml.sax.SAXException;
 import org.xml.sax.helpers.DefaultHandler;
-
-import edu.berkeley.boinc.debug.Debugging;
-import edu.berkeley.boinc.debug.Logging;
 import android.util.Log;
 import android.util.Xml;
+import edu.berkeley.boinc.utils.BOINCDefs;
+import edu.berkeley.boinc.utils.BOINCUtils;
+import edu.berkeley.boinc.utils.Logging;
 
 
 /**
@@ -46,7 +47,6 @@ import android.util.Xml;
  * get_cc_status() becomes getCcStatus() etc.
  */
 public class RpcClient {
-	private static final String TAG = "RpcClient";
 	private static final int CONNECT_TIMEOUT = 30000;      // 30s
 	private static final int READ_TIMEOUT = 15000;         // 15s
 	private static final int READ_BUF_SIZE = 2048;
@@ -63,6 +63,7 @@ public class RpcClient {
 	public static final int PROJECT_NNW     = 4;
 	public static final int PROJECT_ANW     = 5;
 	public static final int PROJECT_DETACH	= 6;
+	public static final int PROJECT_RESET	= 7;
 
 	public static final int RESULT_SUSPEND  = 1;
 	public static final int RESULT_RESUME   = 2;
@@ -142,10 +143,10 @@ public class RpcClient {
 
 	private static final String modeName(int mode) {
 		switch (mode) {
-		case 1: return "<always/>";
-		case 2: return "<auto/>";
-		case 3: return "<never/>";
-		case 4: return "<restore/>";
+		case BOINCDefs.RUN_MODE_ALWAYS: return "<always/>";
+		case BOINCDefs.RUN_MODE_AUTO: return "<auto/>";
+		case BOINCDefs.RUN_MODE_NEVER: return "<never/>";
+		case BOINCDefs.RUN_MODE_RESTORE: return "<restore/>";
 		default: return "";
 		}
 	}
@@ -163,7 +164,7 @@ public class RpcClient {
 	public boolean open(String address, int port) {
 		if (isConnected()) {
 			// Already connected
-			if (Logging.ERROR) Log.e(TAG, "Attempt to connect when already connected");
+			if(edu.berkeley.boinc.utils.Logging.LOGLEVEL <= 4) Log.e(Logging.TAG, "Attempt to connect when already connected");
 			// We better close current connection and reconnect (address/port could be different)
 			close();
 		}
@@ -175,21 +176,21 @@ public class RpcClient {
 			mOutput = new OutputStreamWriter(mSocket.getOutputStream(), "ISO8859_1");
 		}
 		catch (UnknownHostException e) {
-			if (Logging.WARNING) Log.w(TAG, "connect failure: unknown host \"" + address + "\"", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "connect failure: unknown host \"" + address + "\"", e);
 			mSocket = null;
 			return false;
 		}
 		catch (IllegalArgumentException e) {
-			if (Logging.ERROR) Log.e(TAG, "connect failure: illegal argument", e);
+			if(edu.berkeley.boinc.utils.Logging.LOGLEVEL <= 4) Log.e(Logging.TAG, "connect failure: illegal argument", e);
 			mSocket = null;
 			return false;
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "connect failure", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "connect failure", e);
 			mSocket = null;
 			return false;
 		}
-		if (Logging.DEBUG) Log.d(TAG, "open(" + address + ", " + port + ") - Connected successfully");
+		if(Logging.DEBUG) Log.d(Logging.TAG, "open(" + address + ", " + port + ") - Connected successfully");
 		return true;
 	}
 
@@ -205,20 +206,20 @@ public class RpcClient {
 			mInput.close();
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "input close failure", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "input close failure", e);
 		}
 		try {
 			mOutput.close();
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "output close failure", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "output close failure", e);
 		}
 		try {
 			mSocket.close();
-			if (Logging.DEBUG) Log.d(TAG, "close() - Socket closed");
+			if(Logging.DEBUG) Log.d(Logging.TAG, "close() - Socket closed");
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "socket close failure", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "socket close failure", e);
 		}
 		mSocket = null;
 	}
@@ -257,19 +258,19 @@ public class RpcClient {
 			mRequest.setLength(0);
 			Xml.parse(auth2Rsp, new Auth2Parser(mRequest));
 			if (!mRequest.toString().equals("authorized")) {
-				if (Logging.DEBUG) { Log.d(TAG, "authorize() - Failure"); }
+				if(Logging.DEBUG) Log.d(Logging.TAG, "authorize() - Failure");
 				return false;
 			}
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in authorize()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in authorize()", e);
 			return false;
 		}
 		catch (SAXException e) {
-			if (Logging.INFO) Log.i(TAG, "Malformed XML received in authorize()");
+			Log.i(Logging.TAG, "Malformed XML received in authorize()");
 			return false;
 		}
-		if (Logging.DEBUG) Log.d(TAG, "authorize() - Successful");
+		if(Logging.DEBUG) Log.d(Logging.TAG, "authorize() - Successful");
 		return true;
 	}
 
@@ -316,8 +317,8 @@ public class RpcClient {
 	 * @throws IOException if error occurs when sending the request
 	 */
 	protected void sendRequest(String request) throws IOException {
-		if (Debugging.PERFORMANCE) Log.d(TAG, "mRequest.capacity() = " + mRequest.capacity());
-		if (Debugging.DATA) Log.d(TAG, "Sending request: \n" + request.toString());
+		if (Logging.RPC_PERFORMANCE) if(Logging.DEBUG) Log.d(Logging.TAG, "mRequest.capacity() = " + mRequest.capacity());
+		if (Logging.RPC_DATA) if(Logging.DEBUG) Log.d(Logging.TAG, "Sending request: \n" + request.toString());
 		if (mOutput == null)
 			return;
 		mOutput.write("<boinc_gui_rpc_request>\n");
@@ -333,7 +334,7 @@ public class RpcClient {
 	 */
 	protected String receiveReply() throws IOException {
 		mResult.setLength(0);
-		if (Debugging.PERFORMANCE) Log.d(TAG, "mResult.capacity() = " + mResult.capacity());
+		if (Logging.RPC_PERFORMANCE) if(Logging.DEBUG) Log.d(Logging.TAG, "mResult.capacity() = " + mResult.capacity());
 
 		long readStart = System.nanoTime();
 
@@ -355,23 +356,23 @@ public class RpcClient {
 			}
 		} while (true);
 
-		if (Debugging.PERFORMANCE) {
+		if (Logging.RPC_PERFORMANCE) {
 			float duration = (System.nanoTime() - readStart)/1000000000.0F;
 			long bytesCount = mResult.length();
 			if (duration == 0) duration = 0.001F;
-			Log.d(TAG, "Reading from socket took " + duration + " seconds, " + bytesCount + " bytes read (" + (bytesCount / duration) + " bytes/second)");
+			if(Logging.DEBUG) Log.d(Logging.TAG, "Reading from socket took " + duration + " seconds, " + bytesCount + " bytes read (" + (bytesCount / duration) + " bytes/second)");
 		}
 
-		if (Debugging.PERFORMANCE) Log.d(TAG, "mResult.capacity() = " + mResult.capacity());
+		if (Logging.RPC_PERFORMANCE) if(Logging.DEBUG) Log.d(Logging.TAG, "mResult.capacity() = " + mResult.capacity());
 
-		if (Debugging.DATA) {
+		if (Logging.RPC_DATA) {
 			BufferedReader dbr = new BufferedReader(new StringReader(mResult.toString()));
 			String dl;
 			int ln = 0;
 			try {
-				while ((dl = dbr.readLine()) != null) {
+				while ((dl = BOINCUtils.readLineLimit(dbr, 4096)) != null) {
 					++ln;
-					Log.d(TAG, String.format("%4d: %s", ln, dl));
+					if(Logging.DEBUG) Log.d(Logging.TAG, String.format("%4d: %s", ln, dl));
 				}
 			}
 			catch (IOException ioe) {
@@ -400,7 +401,7 @@ public class RpcClient {
 			return versionInfo;
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in exchangeVersions()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in exchangeVersions()", e);
 			return null;
 		}
 	}
@@ -418,7 +419,7 @@ public class RpcClient {
 			return ccStatus;
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in getCcStatus()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in getCcStatus()", e);
 			return null;
 		}
 	}
@@ -435,7 +436,7 @@ public class RpcClient {
 			return transfers;
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in getFileTransfers()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in getFileTransfers()", e);
 			return null;
 		}
 	}
@@ -453,31 +454,33 @@ public class RpcClient {
 			return hostInfo;
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in getHostInfo()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in getHostInfo()", e);
 			return null;
 		}
 	}
 
 	/**
 	 * Performs get_message_count RPC towards BOINC client
-	 * 
+	 * Returns highest seqNo
 	 * @return result of RPC call in case of success, null otherwise
 	 */
 	public synchronized int getMessageCount() {
 		mLastErrorMessage = null;
 		try {
 			sendRequest("<get_message_count/>\n");
-			return MessageCountParser.getSeqno(receiveReply());
+			int seqNo = MessageCountParser.getSeqno(receiveReply());
+			if(Logging.DEBUG) Log.d(Logging.TAG,"RpcClient.getMessageCount returning: " + seqNo);
+			return seqNo;
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in getMessageCount()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in getMessageCount()", e);
 			return -1;
 		}
 	}
 
 	/**
 	 * Performs get_messages RPC towards BOINC client
-	 * 
+	 * Returns client messages that are more recent than seqNo param
 	 * @return result of RPC call in case of success, null otherwise
 	 */
 	public synchronized ArrayList<Message> getMessages(int seqNo) {
@@ -496,10 +499,11 @@ public class RpcClient {
 			}
 			sendRequest(request);
 			ArrayList<Message> messages = MessagesParser.parse(receiveReply());
+			if(messages == null) messages = new ArrayList<Message>(); // do not return null
 			return messages;
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in getMessages()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in getMessages()", e);
 			return null;
 		}
 	}
@@ -517,12 +521,11 @@ public class RpcClient {
 			return projects;
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in getProjectStatus()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in getProjectStatus()", e);
 			return null;
 		}
 	}
 	
-
 	/**
 	 * Performs get_results RPC towards BOINC client (only active results)
 	 * 
@@ -540,7 +543,7 @@ public class RpcClient {
 			return results;
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in getActiveResults()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in getActiveResults()", e);
 			return null;
 		}
 	}
@@ -558,7 +561,7 @@ public class RpcClient {
 			return results;
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in getResults()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in getResults()", e);
 			return null;
 		}
 	}
@@ -576,11 +579,72 @@ public class RpcClient {
 			return result;
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in getState()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in getState()", e);
 			return null;
 		}
 	}
 
+	/**
+	 * Reports the current device state to the BOINC core client,
+	 * if not called frequently, BOINC core client will suspend
+	 * @return true for success, false for failure
+	 */
+	public synchronized boolean reportDeviceStatus(DeviceStatus deviceStatus) {
+		mLastErrorMessage = null;
+		mRequest.setLength(0);
+		mRequest.append("<report_device_status>\n <device_status>\n  <on_ac_power>");
+		mRequest.append(deviceStatus.isOn_ac_power() ? 1 : 0);
+		mRequest.append("</on_ac_power>\n  <on_usb_power>");
+		mRequest.append(deviceStatus.isOn_usb_power() ? 1 : 0);
+		mRequest.append("</on_usb_power>\n  <battery_charge_pct>");
+		mRequest.append(deviceStatus.getBattery_charge_pct());
+		mRequest.append("</battery_charge_pct>\n  <battery_state>");
+		mRequest.append(deviceStatus.getBattery_state());
+		mRequest.append("</battery_state>\n  <battery_temperature_celsius>");
+		mRequest.append(deviceStatus.getBattery_temperature_celcius());
+		mRequest.append("</battery_temperature_celsius>\n  <wifi_online>");
+		mRequest.append(deviceStatus.isWifi_online() ? 1 : 0);
+		mRequest.append("</wifi_online>\n  <user_active>");
+		mRequest.append(deviceStatus.isUser_active() ? 1 : 0);
+		mRequest.append("</user_active>\n </device_status>\n</report_device_status>\n");
+		try {
+			sendRequest(mRequest.toString());
+			SimpleReplyParser parser = SimpleReplyParser.parse(receiveReply());
+			if (parser == null)
+				return false;
+			mLastErrorMessage = parser.getErrorMessage();
+			return parser.result();
+		}
+		catch (IOException e) {
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in networkAvailable()", e);
+			return false;
+		}
+	}
+	
+	/**
+	 * Reports the Android model as host info to the client
+	 * @return true for success, false for failure
+	 */
+	public synchronized boolean setHostInfo(String hostInfo){
+		mLastErrorMessage = null;
+		mRequest.setLength(0);
+		mRequest.append("<set_host_info>\n <host_info>\n  <product_name>");
+		mRequest.append(hostInfo);
+		mRequest.append("</product_name>\n </host_info>\n</set_host_info>\n");
+		try {
+			sendRequest(mRequest.toString());
+			SimpleReplyParser parser = SimpleReplyParser.parse(receiveReply());
+			if (parser == null)
+				return false;
+			mLastErrorMessage = parser.getErrorMessage();
+			return parser.result();
+		}
+		catch (IOException e) {
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in networkAvailable()", e);
+			return false;
+		}
+	}
+
 	/**
 	 * Tells the BOINC core client that a network connection is available,
 	 * and that it should do as much network activity as it can.
@@ -597,7 +661,7 @@ public class RpcClient {
 			return parser.result();
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in networkAvailable()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in networkAvailable()", e);
 			return false;
 		}
 	}
@@ -630,8 +694,11 @@ public class RpcClient {
 			case PROJECT_DETACH:
 				opTag = "project_detach";
 				break;
+			case PROJECT_RESET:
+				opTag = "project_reset";
+				break;
 			default:
-				if (Logging.ERROR) Log.e(TAG, "projectOp() - unsupported operation: " + operation);
+				if(edu.berkeley.boinc.utils.Logging.LOGLEVEL <= 4) Log.e(Logging.TAG, "projectOp() - unsupported operation: " + operation);
 				return false;
 			}
 			String request =
@@ -647,7 +714,7 @@ public class RpcClient {
 			return parser.result();
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in projectOp()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in projectOp()", e);
 			return false;
 		}
 	}
@@ -686,7 +753,7 @@ public class RpcClient {
 			mLastErrorMessage = parser.getErrorMessage();
 			return parser.result();
 		} catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in createAccount()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in createAccount()", e);
 			return false;
 		}
 	}
@@ -703,7 +770,7 @@ public class RpcClient {
 			sendRequest(mRequest.toString());
 			return AccountOutParser.parse(receiveReply());
 		} catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in getCreateAccountPoll()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in getCreateAccountPoll()", e);
 			return null;
 		}
 	}
@@ -715,15 +782,17 @@ public class RpcClient {
 	 */
 	public synchronized boolean lookupAccount(AccountIn accountIn) {
 		try {
+			String id;
+			if(accountIn.email_addr == null || accountIn.email_addr.isEmpty()) id = accountIn.user_name;
+			else id = accountIn.email_addr;
 			mRequest.setLength(0);
-			mRequest.append("<lookup_account>\n   <url>");
+			mRequest.append("<lookup_account>\n <url>");
 			mRequest.append(accountIn.url);
-			mRequest.append("</url>\n   <email_addr>");
-			mRequest.append(accountIn.email_addr);
-			mRequest.append("</email_addr>\n   <passwd_hash>");
-			mRequest.append(getPasswdHash(accountIn.passwd, accountIn.email_addr));
-			mRequest.append("</passwd_hash>\n<lookup_account>\n");
-			
+			mRequest.append("</url>\n <email_addr>");
+			mRequest.append(id.toLowerCase(Locale.US));
+			mRequest.append("</email_addr>\n <passwd_hash>");
+			mRequest.append(getPasswdHash(accountIn.passwd, id.toLowerCase(Locale.US)));
+			mRequest.append("</passwd_hash>\n</lookup_account>\n");
 			sendRequest(mRequest.toString());
 			
 			SimpleReplyParser parser = SimpleReplyParser.parse(receiveReply());
@@ -732,7 +801,7 @@ public class RpcClient {
 			mLastErrorMessage = parser.getErrorMessage();
 			return parser.result();
 		} catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in lookupAccount()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in lookupAccount()", e);
 			return false;
 		}
 	}
@@ -749,7 +818,7 @@ public class RpcClient {
 			sendRequest(mRequest.toString());
 			return AccountOutParser.parse(receiveReply());
 		} catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in getLookupAccountPoll()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in getLookupAccountPoll()", e);
 			return null;
 		}
 	}
@@ -779,7 +848,7 @@ public class RpcClient {
 			mLastErrorMessage = parser.getErrorMessage();
 			return parser.result();
 		} catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in projectAttach()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in projectAttach()", e);
 			return false;
 		}
 	}
@@ -796,7 +865,7 @@ public class RpcClient {
 			sendRequest(mRequest.toString());
 			return ProjectAttachReplyParser.parse(receiveReply());
 		} catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in projectAttachPoll()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in projectAttachPoll()", e);
 			return null;
 		}
 	}
@@ -815,7 +884,7 @@ public class RpcClient {
 			mLastErrorMessage = parser.getErrorMessage();
 			return parser.result();
 		} catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in getProjectConfig()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in getProjectConfig()", e);
 			return false;
 		}
 	}
@@ -828,7 +897,7 @@ public class RpcClient {
 			sendRequest(mRequest.toString());
 			return ProjectConfigReplyParser.parse(receiveReply());
 		} catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in getProjectConfigPoll()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in getProjectConfigPoll()", e);
 			return null;
 		}
 	}
@@ -841,7 +910,7 @@ public class RpcClient {
 			sendRequest(mRequest.toString());
 			return ProjectInfoParser.parse(receiveReply());
 		} catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in getAllProjectsList()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in getAllProjectsList()", e);
 			return null;
 		}
 		
@@ -855,7 +924,7 @@ public class RpcClient {
 			sendRequest(mRequest.toString());
 			return GlobalPreferencesParser.parse(receiveReply());
 		} catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in globalPrefsWorking()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in globalPrefsWorking()", e);
 			return null;
 		}
 	}
@@ -871,7 +940,7 @@ public class RpcClient {
 			receiveReply();
 			return true;
 		} catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in setGlobalPrefsOverride()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in setGlobalPrefsOverride()", e);
 			return false;
 		}
 	}
@@ -881,7 +950,11 @@ public class RpcClient {
 			mRequest.setLength(0);
 			mRequest.append("<set_global_prefs_override>\n<global_preferences>\n  <run_on_batteries>");
 			mRequest.append(globalPrefs.run_on_batteries ? 1 : 0);
-			mRequest.append("</run_on_batteries>\n  <run_gpu_if_user_active>");
+			mRequest.append("</run_on_batteries>\n  <battery_charge_min_pct>");
+			mRequest.append(globalPrefs.battery_charge_min_pct);
+			mRequest.append("</battery_charge_min_pct>\n  <battery_max_temperature>");
+			mRequest.append(globalPrefs.battery_max_temperature);
+			mRequest.append("</battery_max_temperature>\n  <run_gpu_if_user_active>");
 			mRequest.append(globalPrefs.run_gpu_if_user_active ? 1 : 0);
 			mRequest.append("</run_gpu_if_user_active>\n  <run_if_user_active>");
 			mRequest.append(globalPrefs.run_if_user_active ? 1 : 0);
@@ -967,7 +1040,7 @@ public class RpcClient {
 			receiveReply();
 			return true;
 		} catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in setGlobalPrefsOverrideStruct()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in setGlobalPrefsOverrideStruct()", e);
 			return false;
 		}
 	}
@@ -982,7 +1055,7 @@ public class RpcClient {
 			receiveReply();
 			return true;
 		} catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in setGlobalPrefsOverrideStruct()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in setGlobalPrefsOverrideStruct()", e);
 			return false;
 		}
 	}
@@ -1001,7 +1074,7 @@ public class RpcClient {
 			return parser.result();
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in quit()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in quit()", e);
 			return false;
 		}
 	}
@@ -1028,7 +1101,7 @@ public class RpcClient {
 			return parser.result();
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in setNetworkMode()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in setNetworkMode()", e);
 			return false;
 		}
 	}
@@ -1055,7 +1128,7 @@ public class RpcClient {
 			return parser.result();
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in setRunMode()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in setRunMode()", e);
 			return false;
 		}
 	}
@@ -1078,7 +1151,7 @@ public class RpcClient {
 				opTag = "abort_file_transfer";
 				break;
 			default:
-				if (Logging.ERROR) Log.e(TAG, "transferOp() - unsupported operation: " + operation);
+				if(edu.berkeley.boinc.utils.Logging.LOGLEVEL <= 4) Log.e(Logging.TAG, "transferOp() - unsupported operation: " + operation);
 				return false;
 			}
 			mRequest.setLength(0);
@@ -1100,7 +1173,108 @@ public class RpcClient {
 			return parser.result();
 		}
 		catch (IOException e) {
-			if (Logging.WARNING) Log.w(TAG, "error in transferOp()", e);
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in transferOp()", e);
+			return false;
+		}
+	}
+	
+	/**
+	 * Triggers operation on task in BOINC core client
+	 * @param operation operation to be triggered
+	 * @param projectUrl master URL of project
+	 * @param fileName name of the file
+	 * @return true for success, false for failure
+	 */
+	public boolean resultOp(int operation, String projectUrl, String resultName) {
+		try {
+			String opTag;
+			switch (operation) {
+			case RESULT_SUSPEND:
+				opTag = "suspend_result";
+				break;
+			case RESULT_RESUME:
+				opTag = "resume_result";
+				break;
+			case RESULT_ABORT:
+				opTag = "abort_result";
+				break;
+			default:
+				if(edu.berkeley.boinc.utils.Logging.LOGLEVEL <= 4) Log.e(Logging.TAG, "resultOp() - unsupported operation: " + operation);
+				return false;
+			}
+			mRequest.setLength(0);
+			mRequest.append("<");
+			mRequest.append(opTag);
+			mRequest.append(">\n   <project_url>");
+			mRequest.append(projectUrl);
+			mRequest.append("</project_url>\n   <name>");
+			mRequest.append(resultName);
+			mRequest.append("</name>\n</");
+			mRequest.append(opTag);
+			mRequest.append(">\n");
+			sendRequest(mRequest.toString());
+			
+			SimpleReplyParser parser = SimpleReplyParser.parse(receiveReply());
+			if (parser == null)
+				return false;
+			mLastErrorMessage = parser.getErrorMessage();
+			return parser.result();
+		}
+		catch (IOException e) {
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in transferOp()", e);
+			return false;
+		}
+	}
+	
+	public synchronized boolean setCcConfig(String ccConfig) {
+		final String request =
+				"<set_cc_config>\n" +
+						ccConfig + 
+				"\n</set_cc_config>\n";
+			try {
+				sendRequest(request);
+				SimpleReplyParser parser = SimpleReplyParser.parse(receiveReply());
+				if (parser == null)
+					return false;
+				mLastErrorMessage = parser.getErrorMessage();
+				return parser.result();
+			}
+			catch (IOException e) {
+				if(Logging.WARNING) Log.w(Logging.TAG, "error in setRunMode()", e);
+				return false;
+			}
+		
+	}
+	
+	public synchronized String getCcConfig() {
+		//TODO: needs proper parsing
+		try {
+			mRequest.setLength(0);
+			mRequest.append("<get_cc_config/>");
+			
+			sendRequest(mRequest.toString());
+			String reply = receiveReply();
+			Log.d(Logging.TAG, reply);
+			return reply;
+		} catch (IOException e) {
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in getCcConfig()", e);
+			return "";
+		}
+	}
+	
+	public synchronized Boolean readCcConfig() {
+		try {
+			mRequest.setLength(0);
+			mRequest.append("<read_cc_config/>");
+
+			sendRequest(mRequest.toString());
+			SimpleReplyParser parser = SimpleReplyParser.parse(receiveReply());
+			if (parser == null)
+				return false;
+			mLastErrorMessage = parser.getErrorMessage();
+			return parser.result();
+		} catch (IOException e) {
+			if(Logging.WARNING) Log.w(Logging.TAG, "error in readCcConfig()", e);
 			return false;
 		}
 	}
