@@ -33,6 +33,7 @@
 
 #include "client_msgs.h"
 #include "client_state.h"
+#include "cs_notice.h"
 #include "file_names.h"
 #include "filesys.h"
 #include "gui_http.h"
@@ -74,10 +75,11 @@ int ACCT_MGR_OP::do_rpc(
         for (i=0; i<gstate.projects.size(); i++) {
             gstate.projects[i]->detach_ams();
         }
+        ::rss_feeds.update_feed_list();
         return 0;
     }
 
-    canonicalize_master_url(url);
+    canonicalize_master_url(url, sizeof(url));
     if (!valid_master_url(url)) {
         error_num = ERR_INVALID_URL;
         return 0;
@@ -311,6 +313,8 @@ int AM_ACCOUNT::parse(XML_PARSER& xp) {
     return ERR_XML_PARSE;
 }
 
+// parse RPC reply from account manager
+//
 int ACCT_MGR_OP::parse(FILE* f) {
     string message;
     int retval;
@@ -324,6 +328,7 @@ int ACCT_MGR_OP::parse(FILE* f) {
     repeat_sec = 0;
     strcpy(host_venue, "");
     strcpy(ami.opaque, "");
+    ami.no_project_notices = false;
     rss_feeds.clear();
     if (!xp.parse_start("acct_mgr_reply")) return ERR_XML_PARSE;
     while (!xp.get_tag()) {
@@ -391,6 +396,9 @@ int ACCT_MGR_OP::parse(FILE* f) {
             parse_rss_feed_descs(xp, rss_feeds);
             continue;
         }
+        if (xp.parse_bool("no_project_notices", ami.no_project_notices)) {
+            continue;
+        }
         if (log_flags.unparsed_xml) {
             msg_printf(NULL, MSG_INFO,
                 "[unparsed_xml] ACCT_MGR_OP::parse: unrecognized %s",
@@ -488,12 +496,13 @@ void ACCT_MGR_OP::handle_reply(int http_op_retval) {
     }
 
     if (sig_ok) {
-        strcpy(gstate.acct_mgr_info.master_url, ami.master_url);
-        strcpy(gstate.acct_mgr_info.project_name, ami.project_name);
-        strcpy(gstate.acct_mgr_info.signing_key, ami.signing_key);
-        strcpy(gstate.acct_mgr_info.login_name, ami.login_name);
-        strcpy(gstate.acct_mgr_info.password_hash, ami.password_hash);
-        strcpy(gstate.acct_mgr_info.opaque, ami.opaque);
+        safe_strcpy(gstate.acct_mgr_info.master_url, ami.master_url);
+        safe_strcpy(gstate.acct_mgr_info.project_name, ami.project_name);
+        safe_strcpy(gstate.acct_mgr_info.signing_key, ami.signing_key);
+        safe_strcpy(gstate.acct_mgr_info.login_name, ami.login_name);
+        safe_strcpy(gstate.acct_mgr_info.password_hash, ami.password_hash);
+        safe_strcpy(gstate.acct_mgr_info.opaque, ami.opaque);
+        gstate.acct_mgr_info.no_project_notices = ami.no_project_notices;
 
         // process projects
         //
@@ -526,7 +535,7 @@ void ACCT_MGR_OP::handle_reply(int http_op_retval) {
                             if (is_weak_auth(pp->authenticator)
                                 && is_weak_auth(acct.authenticator.c_str())
                             ) {
-                                strcpy(pp->authenticator, acct.authenticator.c_str());
+                                safe_strcpy(pp->authenticator, acct.authenticator.c_str());
                                 msg_printf(pp, MSG_INFO,
                                     "Received new authenticator from account manager"
                                 );
@@ -567,7 +576,7 @@ void ACCT_MGR_OP::handle_reply(int http_op_retval) {
                         if (pp->ams_resource_share >= 0) {
                             pp->ams_resource_share = -1;
                             PROJECT p2;
-                            strcpy(p2.master_url, pp->master_url);
+                            safe_strcpy(p2.master_url, pp->master_url);
                             retval = p2.parse_account_file();
                             if (!retval) {
                                 pp->resource_share = p2.resource_share;
@@ -632,7 +641,7 @@ void ACCT_MGR_OP::handle_reply(int http_op_retval) {
 
         bool read_prefs = false;
         if (strlen(host_venue) && strcmp(host_venue, gstate.main_host_venue)) {
-            strcpy(gstate.main_host_venue, host_venue);
+            safe_strcpy(gstate.main_host_venue, host_venue);
             read_prefs = true;
         }
 
@@ -654,12 +663,16 @@ void ACCT_MGR_OP::handle_reply(int http_op_retval) {
             gstate.read_global_prefs();
         }
 
-        if (got_rss_feeds) {
-            handle_sr_feeds(rss_feeds, &gstate.acct_mgr_info);
-        }
+        handle_sr_feeds(rss_feeds, &gstate.acct_mgr_info);
+
+        // in case no_project_notices changed
+        //
+        ::rss_feeds.update_feed_list();
     }
 
-    strcpy(gstate.acct_mgr_info.previous_host_cpid, gstate.host_info.host_cpid);
+    safe_strcpy(
+        gstate.acct_mgr_info.previous_host_cpid, gstate.host_info.host_cpid
+    );
     if (repeat_sec) {
         gstate.acct_mgr_info.next_rpc_time = gstate.now + repeat_sec;
     } else {
@@ -708,12 +721,14 @@ int ACCT_MGR_INFO::write_info() {
                 "    <next_rpc_time>%f</next_rpc_time>\n"
                 "    <opaque>\n%s\n"
                 "    </opaque>\n"
+                "    <no_project_notices>%d</no_project_notices>\n"
                 "</acct_mgr_login>\n",
                 login_name,
                 password_hash,
                 previous_host_cpid,
                 next_rpc_time,
-                opaque
+                opaque,
+                no_project_notices?1:0
             );
             fclose(p);
         }
@@ -733,6 +748,7 @@ void ACCT_MGR_INFO::clear() {
     nfailures = 0;
     send_gui_rpc_info = false;
     password_error = false;
+    no_project_notices = false;
 }
 
 ACCT_MGR_INFO::ACCT_MGR_INFO() {
@@ -767,6 +783,7 @@ int ACCT_MGR_INFO::parse_login_file(FILE* p) {
             }
             continue;
         }
+        else if (xp.parse_bool("no_project_notices", no_project_notices)) continue;
         if (log_flags.unparsed_xml) {
             msg_printf(NULL, MSG_INFO,
                 "[unparsed_xml] unrecognized %s in acct_mgr_login.xml",
@@ -847,4 +864,3 @@ bool ACCT_MGR_INFO::poll() {
     }
     return false;
 }
-
