@@ -420,14 +420,13 @@ void ACTIVE_TASK_SET::get_memory_usage() {
 // Move it from slot dir to project dir
 //
 int ACTIVE_TASK::move_trickle_file() {
-    char project_dir[MAXPATHLEN], new_path[MAXPATHLEN], old_path[MAXPATHLEN];
+    char new_path[MAXPATHLEN], old_path[MAXPATHLEN];
     int retval;
 
-    get_project_dir(result->project, project_dir, sizeof(project_dir));
     sprintf(old_path, "%s/trickle_up.xml", slot_dir);
     sprintf(new_path,
         "%s/trickle_up_%s_%d.xml",
-        project_dir, result->name, (int)time(0)
+        result->project->project_dir(), result->name, (int)time(0)
     );
     retval = boinc_rename(old_path, new_path);
 
@@ -789,7 +788,7 @@ int ACTIVE_TASK_SET::parse(XML_PARSER& xp) {
 #ifndef SIM
 
 void MSG_QUEUE::init(char* n) {
-    strcpy(name, n);
+    safe_strcpy(name, n);
     last_block = 0;
     msgs.clear();
 }
@@ -898,7 +897,7 @@ int ACTIVE_TASK::handle_upload_files() {
 
     DirScanner dirscan(slot_dir);
     while (dirscan.scan(filename)) {
-        strcpy(buf, filename.c_str());
+        safe_strcpy(buf, filename.c_str());
         if (strstr(buf, UPLOAD_FILE_REQ_PREFIX) == buf) {
             char* p = buf+strlen(UPLOAD_FILE_REQ_PREFIX);
             FILE_INFO* fip = result->lookup_file_logical(p);
@@ -986,29 +985,12 @@ void ACTIVE_TASK_SET::init() {
 
 #endif
 
-static const char* task_state_name(int val) {
-    switch (val) {
-    case PROCESS_UNINITIALIZED: return "UNINITIALIZED";
-    case PROCESS_EXECUTING: return "EXECUTING";
-    case PROCESS_SUSPENDED: return "SUSPENDED";
-    case PROCESS_ABORT_PENDING: return "ABORT_PENDING";
-    case PROCESS_EXITED: return "EXITED";
-    case PROCESS_WAS_SIGNALED: return "WAS_SIGNALED";
-    case PROCESS_EXIT_UNKNOWN: return "EXIT_UNKNOWN";
-    case PROCESS_ABORTED: return "ABORTED";
-    case PROCESS_COULDNT_START: return "COULDNT_START";
-    case PROCESS_QUIT_PENDING: return "QUIT_PENDING";
-    case PROCESS_COPY_PENDING: return "COPY_PENDING";
-    }
-    return "Unknown";
-}
-
 void ACTIVE_TASK::set_task_state(int val, const char* where) {
     _task_state = val;
     if (log_flags.task_debug) {
         msg_printf(result->project, MSG_INFO,
             "[task] task_state=%s for %s from %s",
-            task_state_name(val), result->name, where
+            active_task_state_string(val), result->name, where
         );
     }
 }
