@@ -69,7 +69,7 @@ struct ACTIVE_TASK {
     WORKUNIT* wup;
     APP_VERSION* app_version;
     PROCESS_ID pid;
-	PROCINFO procinfo;
+    PROCINFO procinfo;
 
     // START OF ITEMS SAVED IN STATE FILE
     int _task_state;
@@ -180,14 +180,14 @@ struct ACTIVE_TASK {
     }
 
     ACTIVE_TASK();
-	~ACTIVE_TASK();
+    ~ACTIVE_TASK();
     int init(RESULT*);
     void cleanup_task();
 
     int current_disk_usage(double&);
         // disk used by output files and temp files of this task
     void get_free_slot(RESULT*);
-    int start();         // start a process
+    int start(bool test=false);         // start a process
 
     // Termination stuff.
     // Terminology:
@@ -258,7 +258,7 @@ struct ACTIVE_TASK {
     void upload_notify_app(const FILE_INFO*, const FILE_REF*);
     int copy_output_files();
     int setup_file(FILE_INFO*, FILE_REF&, char*, bool, bool);
-	bool must_copy_file(FILE_REF&, bool);
+    bool must_copy_file(FILE_REF&, bool);
     void write_task_state_file();
     void read_task_state_file();
 
@@ -315,4 +315,6 @@ extern double exclusive_gpu_app_running;
 extern int gpu_suspend_reason;
 extern double non_boinc_cpu_usage;
 
+extern void run_test_app();
+
 #endif
