@@ -64,6 +64,7 @@ using std::vector;
 #include "filesys.h"
 #include "parse.h"
 #include "shmem.h"
+#include "str_replace.h"
 #include "str_util.h"
 #include "util.h"
 
@@ -121,7 +122,7 @@ bool ACTIVE_TASK_SET::poll() {
     }
 
     // Check for finish files every 10 sec.
-    // If we already found a finish file, kill the app;
+    // If we already found a finish file, abort the app;
     // it must be hung somewhere in boinc_finish();
     //
     static double last_finish_check_time = 0;
@@ -131,7 +132,9 @@ bool ACTIVE_TASK_SET::poll() {
             ACTIVE_TASK* atp = active_tasks[i];
             if (atp->task_state() == PROCESS_UNINITIALIZED) continue;
             if (atp->finish_file_time) {
-                atp->kill_task(false);
+                // process is still there 10 sec after it wrote finish file.
+                // abort the job
+                atp->abort_task(EXIT_ABORTED_BY_CLIENT, "finish file present too long");
             } else if (atp->finish_file_present()) {
                 atp->finish_file_time = gstate.now;
             }
@@ -212,14 +215,28 @@ static void kill_app_process(int pid, bool will_restart) {
     if (h == NULL) return;
     TerminateProcess(h, will_restart?0:EXIT_ABORTED_BY_CLIENT);
     CloseHandle(h);
+}
 #else
 static void kill_app_process(int pid, bool) {
+    int retval = 0;
 #ifdef SANDBOX
-    kill_via_switcher(pid);
-#endif
-    kill(pid, SIGKILL);
+    retval = kill_via_switcher(pid);
+    if (retval && log_flags.task_debug) {
+        msg_printf(0, MSG_INFO,
+            "[task] kill_via_switcher() failed: %s",
+            boincerror(retval)
+        );
+    }
 #endif
+    retval = kill(pid, SIGKILL);
+    if (retval && log_flags.task_debug) {
+        msg_printf(0, MSG_INFO,
+            "[task] kill() failed: %s",
+            boincerror(retval)
+        );
+    }
 }
+#endif
 
 static inline void kill_processes(vector<int> pids, bool will_restart) {
     for (unsigned int i=0; i<pids.size(); i++) {
@@ -357,7 +374,7 @@ void ACTIVE_TASK::handle_temporary_exit(
         }
         will_restart = true;
         result->schedule_backoff = gstate.now + backoff;
-        strcpy(result->schedule_backoff_reason, reason);
+        safe_strcpy(result->schedule_backoff_reason, reason);
         set_task_state(PROCESS_UNINITIALIZED, "handle_temporary_exit");
     }
 }
@@ -472,7 +489,7 @@ void ACTIVE_TASK::handle_exited_app(int stat) {
                 set_task_state(PROCESS_UNINITIALIZED, "temporary exit");
                 will_restart = true;
                 result->schedule_backoff = gstate.now + x;
-                strcpy(result->schedule_backoff_reason, buf);
+                safe_strcpy(result->schedule_backoff_reason, buf);
             } else {
                 if (log_flags.task_debug) {
                     msg_printf(result->project, MSG_INFO,
@@ -538,6 +555,11 @@ void ACTIVE_TASK::handle_exited_app(int stat) {
 
     cleanup_task();
 
+    if (gstate.run_test_app) {
+        msg_printf(0, MSG_INFO, "test app finished - exiting");
+        exit(0);
+    }
+
     if (!will_restart) {
         copy_output_files();
         int retval = read_stderr_file();
@@ -573,8 +595,8 @@ bool ACTIVE_TASK::temporary_exit_file_present(double& x, char* buf) {
     } else {
         x = y;
     }
-    fgets(buf, 256, f);     // read the \n
-    fgets(buf, 256, f);
+    (void) fgets(buf, 256, f);     // read the \n
+    (void) fgets(buf, 256, f);
     strip_whitespace(buf);
     fclose(f);
     return true;
@@ -875,6 +897,8 @@ int ACTIVE_TASK::read_stderr_file() {
 // This is called when project prefs change,
 // or when a user file has finished downloading.
 //
+// TODO: get rid of this function
+//
 int ACTIVE_TASK::request_reread_prefs() {
     int retval;
     APP_INIT_DATA aid;
@@ -884,10 +908,12 @@ int ACTIVE_TASK::request_reread_prefs() {
     init_app_init_data(aid);
     retval = write_app_init_file(aid);
     if (retval) return retval;
+#if 0
     graphics_request_queue.msg_queue_send(
         xml_graphics_modes[MODE_REREAD_PREFS],
         app_client_shm.shm->graphics_request
     );
+#endif
     return 0;
 }
 
@@ -1024,22 +1050,39 @@ int ACTIVE_TASK_SET::abort_project(PROJECT* project) {
 }
 
 // suspend all currently running tasks
-// called only from CLIENT_STATE::suspend_tasks(),
 // e.g. because on batteries, time of day, benchmarking, CPU throttle, etc.
 //
 void ACTIVE_TASK_SET::suspend_all(int reason) {
     for (unsigned int i=0; i<active_tasks.size(); i++) {
         ACTIVE_TASK* atp = active_tasks[i];
         if (atp->task_state() != PROCESS_EXECUTING) continue;
-        switch (reason) {
-        case SUSPEND_REASON_CPU_THROTTLE:
+
+        // handle CPU throttling separately
+        //
+        if (reason == SUSPEND_REASON_CPU_THROTTLE) {
             if (atp->result->dont_throttle()) continue;
             // if we're doing CPU throttling,
-            // don't suspend CPU apps that use < 1 CPU
+            // don't suspend apps that use < .5 CPU (like GPU and NCI apps)
             //
-            if (!atp->result->uses_coprocs() && atp->app_version->avg_ncpus < 1) continue;
+            if (atp->app_version->avg_ncpus < .5) continue;
+
             atp->preempt(REMOVE_NEVER);
-            break;
+            continue;;
+        }
+
+#ifdef ANDROID
+        // On Android, remove apps from memory if on batteries
+        // no matter what the reason for suspension.
+        // The message polling in the BOINC runtime system
+        // imposes an overhead which drains the battery
+        //
+        if (gstate.host_info.host_is_running_on_batteries()) {
+            atp->preempt(REMOVE_ALWAYS);
+            continue;
+        }
+#endif
+
+        switch (reason) {
         case SUSPEND_REASON_BENCHMARKS:
             atp->preempt(REMOVE_NEVER);
             break;
@@ -1053,8 +1096,15 @@ void ACTIVE_TASK_SET::suspend_all(int reason) {
             if (atp->result->non_cpu_intensive()) break;
             atp->preempt(REMOVE_NEVER);
             break;
+        case SUSPEND_REASON_BATTERY_OVERHEATED:
+        case SUSPEND_REASON_BATTERY_CHARGING:
+            // these conditions can oscillate, so leave apps in mem
+            //
+            atp->preempt(REMOVE_NEVER);
+            break;
         default:
             atp->preempt(REMOVE_MAYBE_USER);
+            break;
         }
     }
 }
@@ -1399,7 +1449,7 @@ void ACTIVE_TASK::read_task_state_file() {
     FILE* f = fopen(path, "r");
     if (!f) return;
     buf[0] = 0;
-    fread(buf, 1, 4096, f);
+    (void) fread(buf, 1, 4096, f);
     fclose(f);
     buf[4095] = 0;
     double x;
