@@ -86,6 +86,7 @@ Commands:\n\
  --run_benchmarks\n\
  --set_gpu_mode mode duration       set GPU run mode for given duration\n\
    mode = always | auto | never\n\
+ --set_host_info product_name\n\
  --set_network_mode mode duration   set network mode for given duration\n\
    mode = always | auto | never\n\
  --set_proxy_settings\n\
@@ -99,10 +100,6 @@ Commands:\n\
     exit(1);
 }
 
-void show_error(int retval) {
-    fprintf(stderr, "Error %d: %s\n", retval, boincerror(retval));
-}
-
 char* next_arg(int argc, char** argv, int& i) {
     if (i >= argc) {
         fprintf(stderr, "Missing command-line argument\n");
@@ -231,9 +228,9 @@ int main(int argc, char** argv) {
     } else if (!strcmp(cmd, "--task")) {
         RESULT result;
         char* project_url = next_arg(argc, argv, i);
-        strcpy(result.project_url, project_url);
+        safe_strcpy(result.project_url, project_url);
         char* name = next_arg(argc, argv, i);
-        strcpy(result.name, name);
+        safe_strcpy(result.name, name);
         char* op = next_arg(argc, argv, i);
         if (!strcmp(op, "suspend")) {
             retval = rpc.result_op(result, "suspend");
@@ -246,8 +243,8 @@ int main(int argc, char** argv) {
         }
     } else if (!strcmp(cmd, "--project")) {
         PROJECT project;
-        strcpy(project.master_url, next_arg(argc, argv, i));
-        canonicalize_master_url(project.master_url);
+        safe_strcpy(project.master_url, next_arg(argc, argv, i));
+        canonicalize_master_url(project.master_url, sizeof(project.master_url));
         char* op = next_arg(argc, argv, i);
         if (!strcmp(op, "reset")) {
             retval = rpc.project_op(project, "reset");
@@ -276,8 +273,8 @@ int main(int argc, char** argv) {
         }
     } else if (!strcmp(cmd, "--project_attach")) {
         char url[256];
-        strcpy(url, next_arg(argc, argv, i));
-        canonicalize_master_url(url);
+        safe_strcpy(url, next_arg(argc, argv, i));
+        canonicalize_master_url(url, sizeof(url));
         char* auth = next_arg(argc, argv, i);
         retval = rpc.project_attach(url, auth, "");
     } else if (!strcmp(cmd, "--file_transfer")) {
@@ -327,6 +324,12 @@ int main(int argc, char** argv) {
         } else {
             fprintf(stderr, "Unknown op %s\n", op);
         }
+    } else if (!strcmp(cmd, "--set_host_info")) {
+        HOST_INFO h;
+        memset(&h, 0, sizeof(h));
+        char* pn = next_arg(argc, argv, i);
+        safe_strcpy(h.product_name, pn);
+        retval = rpc.set_host_info(h);
     } else if (!strcmp(cmd, "--set_network_mode")) {
         char* op = next_arg(argc, argv, i);
         double duration;
@@ -539,8 +542,8 @@ int main(int argc, char** argv) {
     } else {
         usage();
     }
-    if (retval < 0) {
-        show_error(retval);
+    if (retval) {
+        fprintf(stderr, "Operation failed: %s\n", boincerror(retval));
     }
 
 #if defined(_WIN32) && defined(USE_WINSOCK)
