@@ -26,7 +26,7 @@
 #include <ctime>
 #include <cstdarg>
 #include <cstring>
-#include <math.h>
+#include <cmath>
 #if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
@@ -105,6 +105,7 @@ CLIENT_STATE::CLIENT_STATE()
     exit_after_app_start_secs = 0;
     app_started = 0;
     exit_before_upload = false;
+    run_test_app = false;
     show_projects = false;
     strcpy(detach_project_url, "");
     strcpy(main_host_venue, "");
@@ -133,6 +134,7 @@ CLIENT_STATE::CLIENT_STATE()
     disable_graphics = false;
     cant_write_state_file = false;
     benchmarks_running = false;
+    device_status_time = 0;
 
     rec_interval_start = 0;
     retry_shmem_time = 0;
@@ -225,20 +227,6 @@ const char* rsc_name(int i) {
     return coprocs.coprocs[i].type;
 }
 
-// set no_X_apps for anonymous platform project
-//
-static void check_no_apps(PROJECT* p) {
-    for (int i=0; i<coprocs.n_rsc; i++) {
-        p->no_rsc_apps[i] = true;
-    }
-
-    for (unsigned int i=0; i<gstate.app_versions.size(); i++) {
-        APP_VERSION* avp = gstate.app_versions[i];
-        if (avp->project != p) continue;
-        p->no_rsc_apps[avp->gpu_usage.rsc_type] = false;
-    }
-}
-
 // alert user if any jobs need more RAM than available
 //
 static void check_too_large_jobs() {
@@ -283,6 +271,10 @@ void CLIENT_STATE::set_now() {
     clock_change = false;
     if (x < (now-60)) {
         clock_change = true;
+        msg_printf(NULL, MSG_INFO,
+            "New system time (%.0f) < old system time (%.0f); clearing timeouts",
+            x, now
+        );
         clear_absolute_times();
     }
 
@@ -470,12 +462,15 @@ int CLIENT_STATE::init() {
     //
     parse_account_files_venue();
 
-    // fill in p->no_X_apps for anon platform projects
+    // fill in p->no_X_apps for anon platform projects,
+    // and check no_rsc_apps for others
     //
     for (i=0; i<projects.size(); i++) {
         p = projects[i];
         if (p->anonymous_platform) {
-            check_no_apps(p);
+            p->check_no_apps();
+        } else {
+            p->check_no_rsc_apps();
         }
     }
 
@@ -784,12 +779,16 @@ bool CLIENT_STATE::poll_slow_events() {
 #endif
 
     bool old_user_active = user_active;
+#ifdef ANDROID
+    user_active = device_status.user_active;
+#else
     user_active = !host_info.users_idle(
         check_all_logins, global_prefs.idle_time_to_run
 #ifdef __APPLE__
          , &idletime
 #endif
     );
+#endif
 
     if (user_active != old_user_active) {
         request_schedule_cpus("Idle state change");
@@ -838,7 +837,8 @@ bool CLIENT_STATE::poll_slow_events() {
     if (tasks_restarted) {
         if (suspend_reason) {
             if (!tasks_suspended) {
-                suspend_tasks(suspend_reason);
+                show_suspend_tasks_message(suspend_reason);
+                active_tasks.suspend_all(suspend_reason);
             }
             last_suspend_reason = suspend_reason;
         } else {
@@ -851,10 +851,7 @@ bool CLIENT_STATE::poll_slow_events() {
         //
         first = false;
         if (suspend_reason) {
-            msg_printf(NULL, MSG_INFO,
-                "Suspending computation - %s",
-                suspend_reason_string(suspend_reason)
-            );
+            show_suspend_tasks_message(suspend_reason);
         }
     }
     tasks_suspended = (suspend_reason != 0);
@@ -1615,7 +1612,6 @@ bool CLIENT_STATE::update_results() {
             if (rp->is_upload_done()) {
                 rp->set_ready_to_report();
                 rp->completed_time = gstate.now;
-                rp->project->last_upload_start = 0;
                 rp->set_state(RESULT_FILES_UPLOADED, "CS::update_results");
 
                 // clear backoffs for app's resources;
@@ -1815,7 +1811,6 @@ int CLIENT_STATE::reset_project(PROJECT* project, bool detaching) {
             i--;
         }
     }
-    project->last_upload_start = 0;
 
     // if we're in the middle of a scheduler op to the project, abort it
     //
@@ -2022,6 +2017,10 @@ int CLIENT_STATE::quit_activities() {
 void CLIENT_STATE::check_clock_reset() {
     if (!time_stats.last_update) return;
     if (time_stats.last_update <= now) return;
+    msg_printf(NULL, MSG_INFO,
+        "System clock (%.0f) < state file timestamp (%.0f); clearing timeouts",
+        now, time_stats.last_update
+    );
     clear_absolute_times();
 }
 
@@ -2033,10 +2032,6 @@ void CLIENT_STATE::check_clock_reset() {
 // that we could try to patch up, but it's not clear how.
 //
 void CLIENT_STATE::clear_absolute_times() {
-    msg_printf(NULL, MSG_INFO,
-        "System clock was turned backwards; clearing timeouts"
-    );
-
     exclusive_app_running = 0;
     exclusive_gpu_app_running = 0;
     new_version_check_time = now;
