@@ -98,6 +98,10 @@ struct CLIENT_STATE {
     ACTIVE_TASK_SET active_tasks;
     HOST_INFO host_info;
 
+    // the following used only on Android
+    DEVICE_STATUS device_status;
+    double device_status_time;
+
     VERSION_INFO core_client_version;
     string statefile_platform_name;
     int file_xfer_giveup_period;
@@ -137,6 +141,8 @@ struct CLIENT_STATE {
     char attach_project_auth[256];
     bool exit_before_upload;
         // exit when about to upload a file
+    bool run_test_app;
+        // API test mode
 #ifndef _WIN32
     gid_t boinc_project_gid;
 #endif
@@ -390,7 +396,7 @@ struct CLIENT_STATE {
     void get_disk_shares();
     double allowed_disk_usage(double boinc_total);
     int allowed_project_disk_usage(double&);
-    int suspend_tasks(int reason);
+    void show_suspend_tasks_message(int reason);
     int resume_tasks(int reason=0);
     void read_global_prefs(
         const char* fname = GLOBAL_PREFS_FILE_NAME,
@@ -595,4 +601,5 @@ extern double calculate_exponential_backoff(
 #define ACCT_MGR_MAX_BACKOFF    86400
     // min/max account manager RPC backoff
 
+#define ANDROID_KEEPALIVE_TIMEOUT   30
 #endif
