@@ -33,6 +33,7 @@
 #endif
 #endif
 
+#include "common_defs.h"
 #include "filesys.h"
 #include "parse.h"
 #include "str_util.h"
@@ -60,9 +61,12 @@ double CLIENT_STATE::allowed_disk_usage(double boinc_total) {
 
     limit_pct = host_info.d_total*global_prefs.disk_max_used_pct/100.0;
     limit_min_free = boinc_total + host_info.d_free - global_prefs.disk_min_free_gb*GIGA;
-    limit_abs = global_prefs.disk_max_used_gb*(GIGA);
 
-    double size = min(min(limit_abs, limit_pct), limit_min_free);
+    double size = min(limit_pct, limit_min_free);
+    if (global_prefs.disk_max_used_gb) {
+        limit_abs = global_prefs.disk_max_used_gb*(GIGA);
+        size = min(size, limit_abs);
+    }
     if (size < 0) size = 0;
     return size;
 }
@@ -75,19 +79,18 @@ double CLIENT_STATE::allowed_disk_usage(double boinc_total) {
 // GLOBAL_STATE::total_disk_usage
 //
 int CLIENT_STATE::get_disk_usages() {
-    char buf[256];
     unsigned int i;
     double size;
     PROJECT* p;
     int retval;
+    char buf[MAXPATHLEN];
 
     client_disk_usage = 0;
     total_disk_usage = 0;
     for (i=0; i<projects.size(); i++) {
         p = projects[i];
         p->disk_usage = 0;
-        get_project_dir(p, buf, sizeof(buf));
-        retval = dir_size(buf, size);
+        retval = dir_size(p->project_dir(), size);
         if (!retval) p->disk_usage = size;
     }
 
@@ -110,48 +113,81 @@ int CLIENT_STATE::get_disk_usages() {
     return 0;
 }
 
-// populate PROJECT::disk_share for all projects
+// populate PROJECT::disk_share for all projects,
+// i.e. the max space we should allocate to the project.
+// This is calculated as follows:
+// - each project has a "disk_resource_share" (DRS)
+//   This is the resource share plus .1*(max resource share).
+//   This ensures that backup projects get some disk.
+// - each project as a "desired_disk_usage (DDU)", 
+//   which is either its current usage
+//   or an amount sent from the scheduler.
+// - each project has a "quota": (available space)*(drs/total_drs).
+// - a project is "greedy" if DDU > quota.
+// - if a project is non-greedy, share = quota
+// - X = available space - space used by non-greedy projects
+// - if a project is greedy, share = quota
+//   + X*drs/(total drs of greedy projects)
 //
 void CLIENT_STATE::get_disk_shares() {
     PROJECT* p;
     unsigned int i;
 
-    double rss = 0;
+    // compute disk resource shares
+    //
+    double trs = 0;
+    double max_rs = 0;
     for (i=0; i<projects.size(); i++) {
         p = projects[i];
-        rss += p->resource_share;
-        p->disk_share = p->disk_usage;
+        p->ddu = std::max(p->disk_usage, p->desired_disk_usage);
+        double rs = p->resource_share;
+        trs += rs;
+        if (rs > max_rs) max_rs = rs;
+    }
+    if (trs) {
+        max_rs /= 10;
+        for (i=0; i<projects.size(); i++) {
+            p = projects[i];
+            p->disk_resource_share = p->resource_share + max_rs;
+        }
+    } else {
+        for (i=0; i<projects.size(); i++) {
+            p = projects[i];
+            p->disk_resource_share = 1;
+        }
     }
-    if (!rss) return;
 
-    // a project is "greedy" if it's using more than its share of disk
+    // Compute:
+    // greedy_drs: total disk resource share of greedy projects
+    // non_greedy_ddu: total desired disk usage of non-greedy projects
     //
-    double greedy_rs = 0;
-    double non_greedy_usage = 0;
+    double greedy_drs = 0;
+    double non_greedy_ddu = 0;
     double allowed = allowed_disk_usage(total_disk_usage);
     for (i=0; i<projects.size(); i++) {
         p = projects[i];
-        double rs = p->resource_share/rss;
-        if (p->disk_usage > allowed*rs) {
-            greedy_rs += p->resource_share;
+        p->disk_quota = allowed*p->disk_resource_share/trs;
+        if (p->ddu > p->disk_quota) {
+            greedy_drs += p->disk_resource_share;
         } else {
-            non_greedy_usage += p->disk_usage;
+            non_greedy_ddu += p->ddu;
         }
     }
-    if (!greedy_rs) greedy_rs = 1;      // handle projects w/ zero resource share
 
-    double greedy_allowed = allowed - non_greedy_usage;
+    double greedy_allowed = allowed - non_greedy_ddu;
     if (log_flags.disk_usage_debug) {
         msg_printf(0, MSG_INFO,
             "[disk_usage] allowed %.2fMB used %.2fMB",
-            allowed, total_disk_usage
+            allowed/MEGA, total_disk_usage/MEGA
         );
     }
     for (i=0; i<projects.size(); i++) {
         p = projects[i];
-        double rs = p->resource_share/rss;
-        if (p->disk_usage > allowed*rs) {
-            p->disk_share = greedy_allowed*p->resource_share/greedy_rs;
+        double rs = p->disk_resource_share/trs;
+        if (p->ddu > allowed*rs) {
+            p->disk_share = greedy_allowed*p->disk_resource_share/greedy_drs;
+        } else {
+            p->disk_share = p->disk_quota;
         }
         if (log_flags.disk_usage_debug) {
             msg_printf(p, MSG_INFO,
@@ -162,14 +198,16 @@ void CLIENT_STATE::get_disk_shares() {
     }
 }
 
-// See if we should suspend processing
+// See if we should suspend CPU and/or GPU processing;
+// return the CPU suspend_reason,
+// and if it's zero set gpu_suspend_reason
 //
 int CLIENT_STATE::check_suspend_processing() {
-    if (are_cpu_benchmarks_running()) {
+    if (benchmarks_running) {
         return SUSPEND_REASON_BENCHMARKS;
     }
 
-    if (config.start_delay && now < client_start_time + config.start_delay) {
+    if (config.start_delay && now < time_stats.client_start_time + config.start_delay) {
         return SUSPEND_REASON_INITIAL_DELAY;
     }
 
@@ -211,6 +249,8 @@ int CLIENT_STATE::check_suspend_processing() {
         }
     }
 
+    // CPU throttling
+    //
     if (global_prefs.cpu_usage_limit < 99) {        // round-off?
         static double last_time=0, debt=0;
         double diff = now - last_time;
@@ -225,6 +265,34 @@ int CLIENT_STATE::check_suspend_processing() {
         }
     }
 
+#ifdef ANDROID
+    if (now > device_status_time + ANDROID_KEEPALIVE_TIMEOUT) {
+        return SUSPEND_REASON_NO_GUI_KEEPALIVE;
+    }
+
+    // check for hot battery
+    //
+    if (device_status.battery_state == BATTERY_STATE_OVERHEATED) {
+        return SUSPEND_REASON_BATTERY_OVERHEATED;
+    }
+    if (device_status.battery_temperature_celsius > global_prefs.battery_max_temperature) {
+        return SUSPEND_REASON_BATTERY_OVERHEATED;
+    }
+
+    // on some devices, running jobs can drain the battery even
+    // while it's recharging.
+    // So compute only if 95% charged or more.
+    //
+    int cp = device_status.battery_charge_pct;
+    if (cp >= 0) {
+        if (cp < global_prefs.battery_charge_min_pct) {
+            return SUSPEND_REASON_BATTERY_CHARGING;
+        }
+    }
+#endif
+
+    // CPU is not suspended.  See if GPUs are
+    //
     if (!coprocs.none()) {
         int old_gpu_suspend_reason = gpu_suspend_reason;
         gpu_suspend_reason = 0;
@@ -245,36 +313,57 @@ int CLIENT_STATE::check_suspend_processing() {
             }
         }
 
-        if (log_flags.cpu_sched) {
-            if (old_gpu_suspend_reason && !gpu_suspend_reason) {
-                msg_printf(NULL, MSG_INFO, "[cpu_sched] resuming GPU activity");
-                request_schedule_cpus("GPU resumption");
-            } else if (!old_gpu_suspend_reason && gpu_suspend_reason) {
-                msg_printf(NULL, MSG_INFO, "[cpu_sched] suspending GPU activity");
-                request_schedule_cpus("GPU suspension");
+        if (old_gpu_suspend_reason && !gpu_suspend_reason) {
+            if (log_flags.task) {
+                msg_printf(NULL, MSG_INFO, "Resuming GPU computation");
             }
+            request_schedule_cpus("GPU resumption");
+        } else if (!old_gpu_suspend_reason && gpu_suspend_reason) {
+            if (log_flags.task) {
+                msg_printf(NULL, MSG_INFO, "Suspending GPU computation - %s",
+                    suspend_reason_string(gpu_suspend_reason)
+                );
+            }
+            request_schedule_cpus("GPU suspension");
         }
     }
 
     return 0;
 }
 
-
-void print_suspend_tasks_message(int reason) {
-    msg_printf(NULL, MSG_INFO, "Suspending computation - %s", suspend_reason_string(reason));
-}
-
-
-int CLIENT_STATE::suspend_tasks(int reason) {
+void CLIENT_STATE::show_suspend_tasks_message(int reason) {
     if (reason == SUSPEND_REASON_CPU_THROTTLE) {
         if (log_flags.cpu_sched) {
             msg_printf(NULL, MSG_INFO, "[cpu_sched] Suspending - CPU throttle");
         }
     } else {
-        print_suspend_tasks_message(reason);
+        if (log_flags.task) {
+            msg_printf(NULL, MSG_INFO,
+                "Suspending computation - %s",
+                suspend_reason_string(reason)
+            );
+        }
+        switch (reason) {
+        case SUSPEND_REASON_BATTERY_OVERHEATED:
+            if (log_flags.task) {
+                msg_printf(NULL, MSG_INFO,
+                    "(battery temperature %.1f > limit %.1f Celsius)",
+                    device_status.battery_temperature_celsius,
+                    global_prefs.battery_max_temperature
+                );
+            }
+            break;
+        case SUSPEND_REASON_BATTERY_CHARGING:
+            if (log_flags.task) {
+                msg_printf(NULL, MSG_INFO,
+                    "(battery charge level %.1f%% < threshold %.1f%%",
+                    device_status.battery_charge_pct,
+                    global_prefs.battery_charge_min_pct
+                );
+            }
+            break;
+        }
     }
-    active_tasks.suspend_all(reason);
-    return 0;
 }
 
 int CLIENT_STATE::resume_tasks(int reason) {
@@ -284,7 +373,9 @@ int CLIENT_STATE::resume_tasks(int reason) {
         }
         active_tasks.unsuspend_all();
     } else {
-        msg_printf(NULL, MSG_INFO, "Resuming computation");
+        if (log_flags.task) {
+            msg_printf(NULL, MSG_INFO, "Resuming computation");
+        }
         active_tasks.unsuspend_all();
         request_schedule_cpus("Resuming computation");
     }
@@ -332,6 +423,21 @@ void CLIENT_STATE::check_suspend_network() {
         goto done;
     }
 
+#ifdef ANDROID
+    if (now > device_status_time + ANDROID_KEEPALIVE_TIMEOUT) {
+        file_xfers_suspended = true;
+        if (!recent_rpc) network_suspended = true;
+        network_suspend_reason = SUSPEND_REASON_NO_GUI_KEEPALIVE;
+    }
+    // use only WiFi
+    //
+    if (global_prefs.network_wifi_only && !device_status.wifi_online) {
+        file_xfers_suspended = true;
+        if (!recent_rpc) network_suspended = true;
+        network_suspend_reason = SUSPEND_REASON_WIFI_STATE;
+    }
+#endif
+
     if (global_prefs.daily_xfer_limit_mb && global_prefs.daily_xfer_period_days) {
         double up, down;
         daily_xfer_history.totals(
@@ -439,13 +545,13 @@ int PROJECT::parse_preferences_for_user_files() {
             fip = new FILE_INFO;
             fip->project = this;
             fip->download_urls.add(url);
-            strcpy(fip->name, filename.c_str());
+            safe_strcpy(fip->name, filename.c_str());
             fip->is_user_file = true;
             gstate.file_infos.push_back(fip);
         }
 
         fr.file_info = fip;
-        strcpy(fr.open_name, open_name.c_str());
+        safe_strcpy(fr.open_name, open_name.c_str());
         user_files.push_back(fr);
     }
     return 0;
@@ -503,7 +609,7 @@ void CLIENT_STATE::read_global_prefs(
             //
             PROJECT* p = global_prefs_source_project();
             if (p && strcmp(main_host_venue, p->host_venue)) {
-                strcpy(main_host_venue, p->host_venue);
+                safe_strcpy(main_host_venue, p->host_venue);
                 global_prefs.parse_file(fname, main_host_venue, found_venue);
             }
         }
@@ -553,6 +659,11 @@ void CLIENT_STATE::read_global_prefs(
     }
     if (!global_prefs.run_if_user_active) {
         msg_printf(NULL, MSG_INFO, "   don't compute while active");
+#ifdef ANDROID
+    } else {
+        msg_printf(NULL, MSG_INFO, "   Android: don't compute while active");
+        global_prefs.run_if_user_active = false;
+#endif
     }
     if (!global_prefs.run_gpu_if_user_active) {
         msg_printf(NULL, MSG_INFO, "   don't use GPU while active");
@@ -580,7 +691,7 @@ void CLIENT_STATE::read_global_prefs(
     file_xfers->set_bandwidth_limits(false);
 #endif
     msg_printf(NULL, MSG_INFO,
-        "   (to change preferences, visit the web site of an attached project, or select Preferences in the Manager)"
+        "   (to change preferences, visit a project web site or select Preferences in the Manager)"
     );
     request_schedule_cpus("Prefs update");
     request_work_fetch("Prefs update");
@@ -635,4 +746,3 @@ double CLIENT_STATE::max_available_ram() {
         global_prefs.ram_max_used_busy_frac, global_prefs.ram_max_used_idle_frac
     );
 }
-
