@@ -17,6 +17,7 @@
 
 #ifdef _WIN32
 #include "boinc_win.h"
+#include "win_util.h"
 #else
 #include "config.h"
 #include <cstring>
@@ -27,6 +28,7 @@
 #include "filesys.h"
 #include "md5_file.h"
 #include "parse.h"
+#include "str_replace.h"
 #include "str_util.h"
 #include "util.h"
 
@@ -68,7 +70,8 @@ static bool valid_state_file(const char* fname) {
     return false;
 }
 
-// return true if r0 arrived before r1
+// return true if r0 arrived before r1,
+// with tie-break based on name hash.
 // used to sort result list
 //
 static inline bool arrived_first(RESULT* r0, RESULT* r1) {
@@ -271,7 +274,7 @@ int CLIENT_STATE::parse_state_file_aux(const char* fname) {
                 continue;
             } 
             if (strlen(avp->platform) == 0) {
-                strcpy(avp->platform, get_primary_platform());
+                safe_strcpy(avp->platform, get_primary_platform());
             } else {
                 if (!is_supported_platform(avp->platform)) {
                     // if it's a platform we haven't heard of,
@@ -285,7 +288,7 @@ int CLIENT_STATE::parse_state_file_aux(const char* fname) {
                         avp->platform, get_primary_platform()
                     );
 #endif
-                    strcpy(avp->platform, get_primary_platform());
+                    safe_strcpy(avp->platform, get_primary_platform());
                 }
             }
             if (avp->missing_coproc) {
@@ -361,7 +364,7 @@ int CLIENT_STATE::parse_state_file_aux(const char* fname) {
             // skip for anon platform
             if (!project->anonymous_platform) {
                 if (!strlen(rp->platform) || !is_supported_platform(rp->platform)) {
-                    strcpy(rp->platform, get_primary_platform());
+                    safe_strcpy(rp->platform, get_primary_platform());
                     rp->version_num = latest_version(rp->wup->app, rp->platform);
                 }
             }
@@ -401,7 +404,7 @@ int CLIENT_STATE::parse_state_file_aux(const char* fname) {
         if (xp.match_tag("host_info")) {
 #ifdef SIM
             retval = host_info.parse(xp, false);
-            coprocs = host_info._coprocs;
+            coprocs = host_info.coprocs;
 #else
             retval = host_info.parse(xp, true);
 #endif
@@ -545,8 +548,11 @@ int CLIENT_STATE::parse_state_file_aux(const char* fname) {
     return 0;
 }
 
-// this is called whenever new results are added, namely at startup
-// and after a scheduler RPC
+// this is called whenever new results are added,
+// namely at startup and after a scheduler RPC.
+// Sort results based on (arrival time, name),
+// then set result.index to their position in this order.
+// This determines the order in which results are run.
 //
 void CLIENT_STATE::sort_results() {
     unsigned int i;
@@ -627,7 +633,7 @@ int CLIENT_STATE::write_state_file() {
 #ifdef _WIN32
                         msg_printf(0, MSG_INFO,
                             "Can't delete previous state file; %s",
-                            windows_error_string(win_error_msg, sizeof(win_error_msg))
+                            windows_format_error_string(GetLastError(), win_error_msg, sizeof(win_error_msg))
                         );
 #else
                         msg_printf(0, MSG_INFO,
@@ -646,7 +652,7 @@ int CLIENT_STATE::write_state_file() {
 #ifdef _WIN32
                     msg_printf(0, MSG_INFO,
                         "Can't rename current state file to previous state file; %s",
-                        windows_error_string(win_error_msg, sizeof(win_error_msg))
+                        windows_format_error_string(GetLastError(), win_error_msg, sizeof(win_error_msg))
                     );
 #else
                     msg_printf(0, MSG_INFO, 
@@ -671,7 +677,7 @@ int CLIENT_STATE::write_state_file() {
 #ifdef _WIN32
             msg_printf(0, MSG_INFO,
                 "rename error: %s",
-                windows_error_string(win_error_msg, sizeof(win_error_msg))
+                windows_format_error_string(GetLastError(), win_error_msg, sizeof(win_error_msg))
             );
 #elif defined (__APPLE__)
             if (log_flags.statefile_debug) {
@@ -801,14 +807,13 @@ int CLIENT_STATE::write_state_file_if_needed() {
 //
 void CLIENT_STATE::check_anonymous() {
     unsigned int i;
-    char dir[256], path[256];
+    char path[MAXPATHLEN];
     FILE* f;
     int retval;
 
     for (i=0; i<projects.size(); i++) {
         PROJECT* p = projects[i];
-        get_project_dir(p, dir, sizeof(dir));
-        sprintf(path, "%s/%s", dir, APP_INFO_FILE_NAME);
+        sprintf(path, "%s/%s", p->project_dir(), APP_INFO_FILE_NAME);
         f = fopen(path, "r");
         if (!f) continue;
         msg_printf(p, MSG_INFO,
@@ -832,7 +837,7 @@ void CLIENT_STATE::check_anonymous() {
 // parse a project's app_info.xml (anonymous platform) file
 //
 int CLIENT_STATE::parse_app_info(PROJECT* p, FILE* in) {
-    char buf[256], path[1024];
+    char buf[256], path[MAXPATHLEN];
     MIOFILE mf;
     mf.init_file(in);
     XML_PARSER xp(&mf);
@@ -861,11 +866,11 @@ int CLIENT_STATE::parse_app_info(PROJECT* p, FILE* in) {
             //
             get_pathname(fip, path, sizeof(path));
             if (!boinc_file_exists(path)) {
-                strcpy(buf,
+                safe_strcpy(buf,
                     _("File referenced in app_info.xml does not exist: ")
                 );
                 strcat(buf, fip->name);
-                msg_printf(p, MSG_USER_ALERT, buf);
+                msg_printf(p, MSG_USER_ALERT, "%s", buf);
                 delete fip;
                 continue;
             }
@@ -895,7 +900,7 @@ int CLIENT_STATE::parse_app_info(PROJECT* p, FILE* in) {
                 continue;
             }
             if (strlen(avp->platform) == 0) {
-                strcpy(avp->platform, get_primary_platform());
+                safe_strcpy(avp->platform, get_primary_platform());
             }
             if (link_app_version(p, avp)) {
                 delete avp;
@@ -922,12 +927,7 @@ int CLIENT_STATE::write_state_gui(MIOFILE& f) {
 
     f.printf("<client_state>\n");
 
-#if 1
-    // NOTE: the following stuff is not in CC_STATE.
-    // However, BoincView (which does its own parsing) expects it
-    // to be in the get_state() reply, so leave it in for now
-    //
-    retval = host_info.write(f, true, false);
+    retval = host_info.write(f, true, true);
     if (retval) return retval;
 
     // the following are for compatibility with old managers
@@ -939,12 +939,18 @@ int CLIENT_STATE::write_state_gui(MIOFILE& f) {
         f.printf("<have_ati/>\n");
     }
 
-    retval = time_stats.write(f, false);
-    if (retval) return retval;
+#if 1
+    // NOTE: the following is not in CC_STATE.
+    // However, BoincView (which does its own parsing) expects it
+    // to be in the get_state() reply, so leave it in for now
+    //
     retval = net_stats.write(f);
     if (retval) return retval;
 #endif
 
+    retval = time_stats.write(f, true);
+    if (retval) return retval;
+
     for (j=0; j<projects.size(); j++) {
         PROJECT* p = projects[j];
         retval = p->write_state(f, true);
