@@ -15,6 +15,12 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
+// XIdleTime:
+// Copyright (C) 2011 Universidade Federal de Campina Grande
+// Initial version: Magnus Henoch
+// Contributors: Danny Kukawka, Eivind Magnus Hvidevold
+// LGPL Version of xidletime: https://github.com/rodrigods/xidletime
+
 // There is a reason that having a file called "cpp.h" that includes config.h
 // and some of the C++ header files is bad.  That reason is because there are
 // #defines that alter the behiour of the standard C and C++ headers.  In
@@ -155,10 +161,12 @@ mach_port_t gEventHandle = NULL;
 #endif
 
 #if HAVE_DPMS
+#include <X11/Xlib.h>
 #include <X11/extensions/dpms.h>
 #endif
 
 #if HAVE_XSS
+#include <X11/Xlib.h>
 #include <X11/extensions/scrnsaver.h>
 #endif
 
@@ -222,32 +230,7 @@ bool HOST_INFO::host_is_running_on_batteries() {
     return retval;
 
 #elif ANDROID
-    // using /sys/class/power_supply/*/online
-    // power supplies are both ac and usb!
-    //
-    static FILE *fsysac, *fsysusb;
-    int aconline = 0;
-    int usbonline = 0;
-    bool power_supply_online = false;
-
-    fsysac = fopen("/sys/class/power_supply/ac/online", "r");
-    fsysusb = fopen("/sys/class/power_supply/usb/online", "r");
-
-    if (fsysac) {
-        (void) fscanf(fsysac, "%d", &aconline);
-        fclose(fsysac);
-    }
-
-    if (fsysusb) {
-        (void) fscanf(fsysusb, "%d", &usbonline);
-        fclose(fsysusb);
-    }
-
-    if ((aconline == 1) || (usbonline == 1)) {
-        power_supply_online = true;
-    }
-
-    return !power_supply_online;
+    return !(gstate.device_status.on_ac_power || gstate.device_status.on_usb_power);
 
 #elif LINUX_LIKE_SYSTEM
     static enum {
@@ -443,121 +426,6 @@ bool HOST_INFO::host_is_running_on_batteries() {
 #endif
 }
 
-#ifdef ANDROID
-
-// Get battery state, charge percentage, and temperature
-//
-
-static const char* battery_dirs[] = {
-    "battery",
-    "BAT0",
-    "bq27520",
-    "bq27200-0",
-    "max17042-0",
-    "ds2760-battery.0",
-    NULL
-};
-
-void HOST_INFO::get_battery_status() {
-    static bool first = true;
-    static FILE *fcap, *fhealth, *fstatus, *ftemp;
-    FILE *f;
-    battery_charge_pct = -1;
-    battery_temperature_celsius = 0;
-    char buf[256];
-    char health[256];
-    char status[256];
-
-    strcpy(health, "");
-    strcpy(status, "");
-
-    if (first) {
-        first = false;
-        int i = 0;
-
-        // Find the battery location for this device.
-        // matszpk has already collected a bunch of battery locations.
-        //
-        for (i = 0; battery_dirs[i] != NULL; i++) {
-            snprintf(buf, sizeof(buf), "/sys/class/power_supply/%s/present",
-                battery_dirs[i]
-            );
-            f = fopen(buf, "r");
-            if (f != NULL) {
-                fclose(f);
-                break;
-            }
-        }
-
-        snprintf(buf, sizeof(buf), "/sys/class/power_supply/%s/capacity",
-            battery_dirs[i]
-        );
-        fcap = fopen(buf, "r");
-        if (fcap) setbuf(fcap, NULL);
-
-        snprintf(buf, sizeof(buf), "/sys/class/power_supply/%s/health",
-            battery_dirs[i]
-        );
-        fhealth = fopen(buf, "r");
-        if (fhealth) setbuf(fhealth, NULL);
-
-        snprintf(buf, sizeof(buf), "/sys/class/power_supply/%s/status",
-            battery_dirs[i]
-        );
-        fstatus = fopen(buf, "r");
-        if (fstatus) setbuf(fstatus, NULL);
-
-        snprintf(buf, sizeof(buf), "/sys/class/power_supply/%s/temp",
-            battery_dirs[i]
-        );
-        ftemp = fopen(buf, "r");
-        if (ftemp) {
-            setbuf(ftemp, NULL);
-        } else {
-            snprintf(buf, sizeof(buf), "/sys/class/power_supply/%s/batt_temp",
-                battery_dirs[i]
-            );
-            ftemp = fopen(buf, "r");
-            if (ftemp) setbuf(ftemp, NULL);
-        }
-    }
-
-    if (fcap) {
-        rewind(fcap);
-        fscanf(fcap, "%d", &battery_charge_pct);
-    }
-
-    if (fhealth) {
-        rewind(fhealth);
-        fgets(health, sizeof(health), fhealth);
-    }
-
-    if (fstatus) {
-        rewind(fstatus);
-        fgets(status, sizeof(status), fstatus);
-    }
-
-    battery_state = BATTERY_STATE_UNKNOWN;
-    if (strstr(health, "Overheat")) {
-        battery_state = BATTERY_STATE_OVERHEATED;
-    } else if (strstr(status, "Not charging")) {
-        battery_state = BATTERY_STATE_DISCHARGING;
-    } else if (strstr(status, "Charging")) {
-        battery_state = BATTERY_STATE_CHARGING;
-    } else if (strstr(status, "Full")) {
-        battery_state = BATTERY_STATE_FULL;
-    }
-
-    if (ftemp) {
-        int x;
-        rewind(ftemp);
-        if (fscanf(ftemp, "%d", &x) == 1) {
-            battery_temperature_celsius = x/10.;
-        }
-    }
-}
-#endif
-
 #if LINUX_LIKE_SYSTEM
 static void parse_meminfo_linux(HOST_INFO& host) {
     char buf[256];
@@ -757,7 +625,7 @@ static void parse_cpuinfo_linux(HOST_INFO& host) {
             }
         }
     }
-    strcpy(model_buf, host.p_model);
+    safe_strcpy(model_buf, host.p_model);
     if (family>=0 || model>=0 || stepping>0) {
         strcat(model_buf, " [");
         if (family>=0) {
@@ -775,10 +643,10 @@ static void parse_cpuinfo_linux(HOST_INFO& host) {
         strcat(model_buf, "]");
     }
     if (strlen(features)) {
-        strlcpy(host.p_features, features, sizeof(host.p_features));
+        safe_strcpy(host.p_features, features);
     }
 
-    strlcpy(host.p_model, model_buf, sizeof(host.p_model));
+    safe_strcpy(host.p_model, model_buf);
     fclose(f);
 }
 #endif  // LINUX_LIKE_SYSTEM
@@ -889,6 +757,12 @@ static void get_cpu_info_maxosx(HOST_INFO& host) {
             *out++ = *in;
         }
     } while (*in++);
+
+    // This returns an Apple hardware model designation such as "MacPro3,1".
+    // One source for converting this to a common model name is:
+    // <http://www.everymac.com/systems/by_capability/mac-specs-by-machine-model-machine-id.html>
+    len = sizeof(host.product_name);
+    sysctlbyname("hw.model", host.product_name, &len, NULL, 0);
 }
 #endif
 
@@ -1338,7 +1212,7 @@ int HOST_INFO::get_virtualbox_version() {
     FILE* fd;
 
 #if LINUX_LIKE_SYSTEM
-    strcpy(path, "/usr/lib/virtualbox/VBoxManage");
+    safe_strcpy(path, "/usr/lib/virtualbox/VBoxManage");
 #elif defined( __APPLE__)
     FSRef theFSRef;
     OSStatus status = noErr;
@@ -1370,11 +1244,12 @@ int HOST_INFO::get_virtualbox_version() {
 #endif
         fd = popen(cmd, "r");
         if (fd) {
-            fgets(virtualbox_version, sizeof(virtualbox_version), fd);
-            newlinePtr = strchr(virtualbox_version, '\n');
-            if (newlinePtr) *newlinePtr = '\0';
-            newlinePtr = strchr(virtualbox_version, '\r');
-            if (newlinePtr) *newlinePtr = '\0';
+            if (fgets(virtualbox_version, sizeof(virtualbox_version), fd)) {
+                newlinePtr = strchr(virtualbox_version, '\n');
+                if (newlinePtr) *newlinePtr = '\0';
+                newlinePtr = strchr(virtualbox_version, '\r');
+                if (newlinePtr) *newlinePtr = '\0';
+            }
             pclose(fd);
         }
     }
@@ -1472,7 +1347,9 @@ int HOST_INFO::get_host_info() {
 ///////////// p_ncpus /////////////////
 
 // sysconf not working on OS2
-#if defined(_SC_NPROCESSORS_ONLN) && !defined(__EMX__) && !defined(__APPLE__)
+#if defined(ANDROID)
+    p_ncpus = sysconf(_SC_NPROCESSORS_CONF);
+#elif defined(_SC_NPROCESSORS_ONLN) && !defined(__EMX__) && !defined(__APPLE__)
     p_ncpus = sysconf(_SC_NPROCESSORS_ONLN);
 #elif defined(HAVE_SYS_SYSCTL_H) && defined(CTL_HW) && defined(HW_NCPU)
     // Get number of CPUs
@@ -1973,6 +1850,9 @@ bool xss_idle(long idle_treshold) {
         idle_time = xssInfo->idle;
 
 #if HAVE_DPMS
+        // XIdleTime Detection
+        // See header for location and copywrites.
+        //
         int dummy;
         CARD16 standby, suspend, off;
         CARD16 state;
@@ -2006,7 +1886,6 @@ bool xss_idle(long idle_treshold) {
                           break;
                     }
                 }
-
             } 
         }
 #endif
