@@ -388,7 +388,7 @@ typedef void (WINAPI *PGNSI)(LPSYSTEM_INFO);
 typedef BOOL (WINAPI *PGPI)(DWORD, DWORD, DWORD, DWORD, PDWORD);
 
 int get_os_information(
-    char* os_name, int /*os_name_size*/, char* os_version, int os_version_size
+    char* os_name, int os_name_size, char* os_version, int os_version_size
 ) {
     // This code snip-it was copied straight out of the MSDN Platform SDK
     //   Getting the System Version example and modified to dump the output
@@ -432,16 +432,16 @@ int get_os_information(
 
 
     // Windows is a Microsoft OS
-    strcpy(os_name, "Microsoft ");
+    strlcpy(os_name, "Microsoft ", os_name_size);
 
     switch (osvi.dwPlatformId) {
         case VER_PLATFORM_WIN32_NT:
 
 			if ( osvi.dwMajorVersion == 6 && osvi.dwMinorVersion == 3 ) {
                 if( osvi.wProductType == VER_NT_WORKSTATION ) {
-                    strcat(os_name, "Windows Blue");
+                    strcat(os_name, "Windows 8.1");
                 } else {
-                    strcat(os_name, "Windows Blue Server");
+                    strcat(os_name, "Windows Server 2012 R2");
                 }
                 pGPI( 6, 3, 0, 0, &dwType);
             }
