@@ -34,6 +34,7 @@
 #include "error_numbers.h"
 #include "filesys.h"
 #include "parse.h"
+#include "str_replace.h"
 #include "str_util.h"
 
 #include "client_state.h"
@@ -53,7 +54,7 @@ static void show_flag(char* buf, bool flag, const char* flag_name) {
     if (!flag) return;
     int n = (int)strlen(buf);
     if (!n) {
-        strcpy(buf, flag_name);
+        strlcpy(buf, flag_name, 256);
         return;
     }
     strcat(buf, ", ");
@@ -116,22 +117,25 @@ static void show_gpu_ignore(vector<int>& devs, int rt) {
     }
 }
 
+// Show GPU exclusions in event log.
+// Don't show errors - they were already shown when we parsed the config file
+//
 static void show_exclude_gpu(EXCLUDE_GPU& e) {
     char t[256], app_name[256], dev[256];
     PROJECT *p = gstate.lookup_project(e.url.c_str());
     if (!p) return;
     if (e.type.empty()) {
-        strcpy(t, "all");
+        safe_strcpy(t, "all");
     } else {
-        strcpy(t, e.type.c_str());
+        safe_strcpy(t, e.type.c_str());
     }
     if (e.appname.empty()) {
-        strcpy(app_name, "all");
+        safe_strcpy(app_name, "all");
     } else {
-        strcpy(app_name, e.appname.c_str());
+        safe_strcpy(app_name, e.appname.c_str());
     }
     if (e.device_num < 0) {
-        strcpy(dev, "all");
+        safe_strcpy(dev, "all");
     } else {
         sprintf(dev, "%d", e.device_num);
     }
@@ -139,27 +143,6 @@ static void show_exclude_gpu(EXCLUDE_GPU& e) {
         "Config: excluded GPU.  Type: %s.  App: %s.  Device: %s",
         t, app_name, dev
     );
-    if (!e.appname.empty()) {
-        APP* app = gstate.lookup_app(p, e.appname.c_str());
-        if (!app) {
-            string app_list;
-            for (unsigned int i=0; i<gstate.apps.size(); i++) {
-                app = gstate.apps[i];
-                if (app->project != p) continue;
-				if (!app_list.empty()) {
-					app_list += ", ";
-				}
-                app_list += "'";
-                app_list += app->name;
-                app_list += "'";
-            }
-            msg_printf(p, MSG_USER_ALERT,
-                "A GPU exclusion in your cc_config.xml file specifies a non-existent application '%s'.  Existing applications: %s",
-                e.appname.c_str(),
-                app_list.c_str()
-            );
-        }
-    }
 }
 
 // Print config info.
@@ -191,6 +174,15 @@ void CONFIG::show() {
     if (fetch_minimal_work) {
         msg_printf(NULL, MSG_INFO, "Config: fetch minimal work");
     }
+    if (max_event_log_lines != DEFAULT_MAX_EVENT_LOG_LINES) {
+        if (max_event_log_lines) {
+            msg_printf(NULL, MSG_INFO,
+                "Config: event log limit %d lines", max_event_log_lines
+            );
+        } else {
+            msg_printf(NULL, MSG_INFO, "Config: event log limit disabled");
+        }
+    }
     if (fetch_on_update) {
         msg_printf(NULL, MSG_INFO, "Config: fetch on update");
     }
@@ -293,6 +285,9 @@ int CONFIG::parse_options_client(XML_PARSER& xp) {
             downcase_string(client_download_url);
             continue;
         }
+        if (xp.parse_string("client_new_version_text", client_new_version_text)) {
+            continue;
+        }
         if (xp.parse_string("client_version_check_url", client_version_check_url)) {
             downcase_string(client_version_check_url);
             continue;
@@ -376,6 +371,7 @@ int CONFIG::parse_options_client(XML_PARSER& xp) {
             ignore_gpu_instance[PROC_TYPE_INTEL_GPU].push_back(n);
             continue;
         }
+        if (xp.parse_int("max_event_log_lines", max_event_log_lines)) continue;
         if (xp.parse_int("max_file_xfers", max_file_xfers)) continue;
         if (xp.parse_int("max_file_xfers_per_project", max_file_xfers_per_project)) continue;
         if (xp.parse_int("max_stderr_file_size", max_stderr_file_size)) continue;
@@ -526,7 +522,12 @@ int read_config_file(bool init, const char* fname) {
 #ifdef _WIN32
             _chdir(config.data_dir);
 #else
-            chdir(config.data_dir);
+            if (chdir(config.data_dir)) {
+                msg_printf(NULL, MSG_INFO,
+                    "Couldn't change to config.data_dir"
+                );
+                return ERR_OPENDIR;
+            }
 #endif
         }
     } else {
@@ -563,6 +564,17 @@ void process_gpu_exclusions() {
             );
             continue;
         }
+        if (!eg.appname.empty()) {
+            APP* app = gstate.lookup_app(p, eg.appname.c_str());
+            if (!app) {
+                msg_printf(p, MSG_USER_ALERT,
+                    "A GPU exclusion in your cc_config.xml file refers to an unknown application '%s'.  Known applications: %s",
+                    eg.appname.c_str(),
+                    app_list_string(p).c_str()
+                );
+                continue;
+            }
+        }
         if (!eg.type.empty()) {
             bool found = false;
             string types;
@@ -570,6 +582,7 @@ void process_gpu_exclusions() {
                 COPROC& cp = coprocs.coprocs[k];
                 if (eg.type == cp.type) {
                     found = true;
+                    rsc_work_fetch[k].has_exclusions = true;
                     break;
                 }
                 types += " " + string(cp.type);
@@ -581,15 +594,9 @@ void process_gpu_exclusions() {
                 );
                 continue;
             }
-            if (!eg.appname.empty()) {
-                APP* app = gstate.lookup_app(p, eg.appname.c_str());
-                if (!app) {
-                    msg_printf(p, MSG_USER_ALERT,
-                        "Nonexistent app '%s' in GPU exclusion",
-                        eg.appname.c_str()
-                    );
-                    continue;
-                }
+        } else {
+            for (int k=1; k<coprocs.n_rsc; k++) {
+                rsc_work_fetch[k].has_exclusions = true;
             }
         }
     }
@@ -607,7 +614,6 @@ void process_gpu_exclusions() {
         for (int k=1; k<coprocs.n_rsc; k++) {
             COPROC& cp = coprocs.coprocs[k];
             int all_instances = (1<<cp.count)-1;  // bitmap of 1 for all inst
-            p->rsc_pwf[k].non_excluded_instances = all_instances;
             for (j=0; j<config.exclude_gpus.size(); j++) {
                 EXCLUDE_GPU& eg = config.exclude_gpus[j];
                 if (!eg.type.empty() && (eg.type != cp.type)) continue;
@@ -639,12 +645,20 @@ void process_gpu_exclusions() {
                 }
             }
 
+            bool found = false;
             p->rsc_pwf[k].non_excluded_instances = 0;
             for (a=0; a<gstate.apps.size(); a++) {
                 APP* app = gstate.apps[a];
                 if (app->project != p) continue;
+                found = true;
                 p->rsc_pwf[k].non_excluded_instances |= app->non_excluded_instances[k];
             }
+            // if project has no apps yet (for some reason)
+            // assume it can use all instances
+            //
+            if (!found) {
+                p->rsc_pwf[k].non_excluded_instances = all_instances;
+            }
 
             // compute ncoprocs_excluded as the number of instances
             // excluded for at least 1 app
