@@ -468,9 +468,10 @@ void RESULT::append_log_record() {
     job_log_filename(*project, filename, sizeof(filename));
     FILE* f = fopen(filename, "ab");
     if (!f) return;
-    fprintf(f, "%.0f ue %f ct %f fe %.0f nm %s et %f\n",
+    fprintf(f, "%.0f ue %f ct %f fe %.0f nm %s et %f es %d\n",
         gstate.now, estimated_runtime_uncorrected(), final_cpu_time,
-        wup->rsc_fpops_est, name, final_elapsed_time
+        wup->rsc_fpops_est, name, final_elapsed_time,
+        exit_status
     );
     fclose(f);
 }
@@ -538,12 +539,22 @@ double RESULT::estimated_runtime() {
     if (!project->dont_use_dcf) {
         x *= project->duration_correction_factor;
     }
-	return x;
+    return x;
 }
 
 double RESULT::estimated_runtime_remaining() {
     if (computing_done()) return 0;
     ACTIVE_TASK* atp = gstate.lookup_active_task_by_result(this);
+    if (app->non_cpu_intensive) {
+        if (atp && atp->fraction_done>0) {
+            double est_dur = atp->fraction_done_elapsed_time / atp->fraction_done;
+            double x = est_dur - atp->elapsed_time;
+            if (x <= 0) x = 1;
+            return x;
+        }
+        return 0;
+    }
+
     if (atp) {
 #ifdef SIM
         return sim_flops_left/avp->flops;
