@@ -33,12 +33,18 @@
 
 #include "work_fetch.h"
 
+#if 0
+#define DEBUG(x) x
+#else
+#define DEBUG(X)
+#endif
+
 using std::vector;
 
 RSC_WORK_FETCH rsc_work_fetch[MAX_RSC];
 WORK_FETCH work_fetch;
 
-inline bool dont_fetch(PROJECT* p, int rsc_type) {
+static inline bool dont_fetch(PROJECT* p, int rsc_type) {
     if (p->no_rsc_pref[rsc_type]) return true;
     if (p->no_rsc_config[rsc_type]) return true;
     if (p->no_rsc_apps[rsc_type]) return true;
@@ -81,13 +87,14 @@ void set_no_rsc_config() {
     }
 }
 
-// does the project have a downloading or runnable job?
+// does the (NCI) project have a job that's running or uploading?
+// (don't request another job from NCI project if so)
 //
-static bool has_a_job(PROJECT* p) {
+static bool has_a_job_in_progress(PROJECT* p) {
     for (unsigned int j=0; j<gstate.results.size(); j++) {
         RESULT* rp = gstate.results[j];
         if (rp->project != p) continue;
-        if (rp->state() <= RESULT_FILES_DOWNLOADED) {
+        if (rp->state() < RESULT_FILES_UPLOADED) {
             return true;
         }
     }
@@ -180,147 +187,10 @@ static bool wacky_dcf(PROJECT* p) {
     return (dcf < 0.02 || dcf > 80.0);
 }
 
-// If this resource is below min buffer level,
-// return the highest-priority project that may have jobs for it.
-//
-// It the resource has instanced starved because of exclusions,
-// return the highest-priority project that may have jobs
-// and doesn't exclude those instances.
-//
-// Only choose a project if the buffer is below min level;
-// if strict_hyst is true, relax this to max level
-//
-// If backoff_exempt_project is non-NULL,
-// don't enforce resource backoffs for that project;
-// this is for when we're going to do a scheduler RPC anyway
-// and we're deciding whether to piggyback a work request
-//
-PROJECT* RSC_WORK_FETCH::choose_project_hyst(
-    bool strict_hyst,
-    PROJECT* backoff_exempt_project
-) {
-    PROJECT* pbest = NULL;
-    bool buffer_low = true;
-    if (strict_hyst) {
-        if (saturated_time > gstate.work_buf_min()) buffer_low = false;
-    } else {
-        if (saturated_time > gstate.work_buf_total()) buffer_low = false;
-    }
-
-    if (log_flags.work_fetch_debug) {
-        msg_printf(0, MSG_INFO,
-            "[work_fetch] choose_project() for %s: buffer_low: %s; sim_excluded_instances %d\n",
-            rsc_name(rsc_type), buffer_low?"yes":"no", sim_excluded_instances
-        );
-    }
-
-    if (!buffer_low && !sim_excluded_instances) return NULL;
-
-    for (unsigned i=0; i<gstate.projects.size(); i++) {
-        PROJECT* p = gstate.projects[i];
-
-        // check whether we can fetch work of any type from this project
-        //
-        if (p->pwf.cant_fetch_work_reason) {
-            //msg_printf(p, MSG_INFO, "skip: cfwr %d", p->pwf.cant_fetch_work_reason);
-            continue;
-        }
-
-        // see whether work fetch for this resource is banned
-        // by prefs, config, project, or acct mgr
-        //
-        if (dont_fetch(p, rsc_type)) {
-            //msg_printf(p, MSG_INFO, "skip: dont_fetch");
-            continue;
-        }
-
-        // check backoff
-        //
-        if (p != backoff_exempt_project) {
-            if (project_state(p).backoff_time > gstate.now) {
-                //msg_printf(p, MSG_INFO, "skip: backoff");
-                continue;
-            }
-        }
-
-        // if project has zero resource share,
-        // only fetch work if a device is idle
-        //
-        if (p->resource_share == 0 && nidle_now == 0) {
-            //msg_printf(p, MSG_INFO, "skip: zero share");
-            continue;
-        }
-
-        // if project has excluded GPUs of this type,
-        // we need to avoid fetching work just because there's an idle instance
-        // or a shortfall;
-        // fetching work might not alleviate either of these,
-        // and we'd end up fetching unbounded work.
-        // At the same time, we want to respect work buf params if possible.
-        //
-        // Current policy:
-        // don't fetch work if remaining time of this project's jobs
-        // exceeds work_buf_min * (#usable instances / #instances)
-        //
-        // TODO: THIS IS FAIRLY CRUDE. Making it smarter would require
-        // computing shortfall etc. on a per-project basis
-        //
-        int nexcl = p->rsc_pwf[rsc_type].ncoprocs_excluded;
-        if (rsc_type && nexcl) {
-            int n_not_excluded = ninstances - nexcl;
-            if (p->rsc_pwf[rsc_type].queue_est > (gstate.work_buf_min() * n_not_excluded)/ninstances) {
-                //msg_printf(p, MSG_INFO, "skip: too much work");
-                continue;
-            }
-        }
-
-        RSC_PROJECT_WORK_FETCH& rpwf = project_state(p);
-        if (rpwf.anon_skip) {
-            //msg_printf(p, MSG_INFO, "skip: anon");
-            continue;
-        }
-
-        // if we're sending work only because of exclusion starvation,
-        // make sure this project can use the starved instances
-        //
-        if (!buffer_low) {
-            if ((sim_excluded_instances & rpwf.non_excluded_instances) == 0) {
-                //msg_printf(p, MSG_INFO, "skip: excl");
-                continue;
-            }
-        }
-
-        if (pbest) {
-            if (pbest->sched_priority > p->sched_priority) {
-                //msg_printf(p, MSG_INFO, "skip: prio");
-                continue;
-            }
-        }
-        pbest = p;
-    }
-    if (!pbest) {
-        if (log_flags.work_fetch_debug) {
-            msg_printf(0, MSG_INFO,
-                "[work_fetch] no eligible project for %s",
-                rsc_name(rsc_type)
-            );
-        }
-        return NULL;
-    }
-    work_fetch.clear_request();
-    if (buffer_low) {
-        work_fetch.set_all_requests_hyst(pbest, rsc_type);
-    } else {
-        set_request_excluded(pbest);
-    }
-    return pbest;
-}
-
 // request this project's share of shortfall and instances.
 // don't request anything if project is backed off.
 //
 void RSC_WORK_FETCH::set_request(PROJECT* p) {
-    if (dont_fetch(p, rsc_type)) return;
 
     // if backup project, fetch 1 job per idle instance
     //
@@ -335,8 +205,6 @@ void RSC_WORK_FETCH::set_request(PROJECT* p) {
         return;
     }
     RSC_PROJECT_WORK_FETCH& w = project_state(p);
-    if (!w.may_have_work) return;
-    if (w.anon_skip) return;
     double non_excl_inst = ninstances - w.ncoprocs_excluded;
     if (shortfall) {
         if (wacky_dcf(p)) {
@@ -382,10 +250,11 @@ void RSC_WORK_FETCH::set_request_excluded(PROJECT* p) {
     int inst_mask = sim_excluded_instances & pwf.non_excluded_instances;
     int n = 0;
     for (int i=0; i<ninstances; i++) {
-        if ((i<<i) & inst_mask) {
+        if ((1<<i) & inst_mask) {
             n++;
         }
     }
+    DEBUG(msg_printf(p, MSG_INFO, "set_request_excluded() %d %d %d", sim_excluded_instances, pwf.non_excluded_instances, n));
     req_instances = n;
     if (p->resource_share == 0 || config.fetch_minimal_work) {
         req_secs = 1;
@@ -401,6 +270,7 @@ void RSC_WORK_FETCH::print_state(const char* name) {
         shortfall, nidle_now, saturated_time,
         busy_time_estimator.get_busy_time()
     );
+    //msg_printf(0, MSG_INFO, "[work_fetch] sim used inst %d sim excl inst %d", sim_used_instances, sim_excluded_instances);
     for (unsigned int i=0; i<gstate.projects.size(); i++) {
         char buf[256];
         PROJECT* p = gstate.projects[i];
@@ -483,6 +353,7 @@ void WORK_FETCH::rr_init() {
     }
 }
 
+#if 0
 // if the given project is highest-priority among the projects
 // eligible for the resource, set request fields
 //
@@ -546,6 +417,7 @@ void WORK_FETCH::set_all_requests(PROJECT* p) {
         }
     }
 }
+#endif
 
 void WORK_FETCH::print_state() {
     msg_printf(0, MSG_INFO, "[work_fetch] ------- start work fetch state -------");
@@ -586,44 +458,90 @@ void WORK_FETCH::clear_request() {
     }
 }
 
-// we're going to contact this project for reasons other than work fetch;
-// decide if we should piggy-back a work fetch request.
+bool WORK_FETCH::requested_work() {
+    for (int i=0; i<coprocs.n_rsc; i++) {
+        if (rsc_work_fetch[i].req_secs) return true;
+    }
+    return false;
+}
+
+// We're going to contact this project for reasons other than work fetch
+// (e.g., to report completed results, or at user request).
+// Decide if we should "piggyback" a work fetch request.
 //
 void WORK_FETCH::piggyback_work_request(PROJECT* p) {
+    DEBUG(msg_printf(p, MSG_INFO, "piggyback_work_request()");)
     clear_request();
     if (config.fetch_minimal_work && gstate.had_or_requested_work) return;
-    if (p->dont_request_more_work) return;
     if (p->non_cpu_intensive) {
-        if (!has_a_job(p)) {
+        if (!has_a_job_in_progress(p) && !p->dont_request_more_work) {
             rsc_work_fetch[0].req_secs = 1;
         }
         return;
     }
 
-    // if project was updated from manager and config says so,
-    // always fetch work if needed
-    //
-    if (p->sched_rpc_pending && config.fetch_on_update) {
-        set_all_requests_hyst(p, -1);
+    setup();
+
+    switch (p->pwf.cant_fetch_work_reason) {
+    case 0:
+    case CANT_FETCH_WORK_MIN_RPC_TIME:
+        break;
+    default:
         return;
     }
-    compute_cant_fetch_work_reason();
-    PROJECT* bestp = choose_project(false, p);
-    if (p != bestp) {
-        if (p->pwf.cant_fetch_work_reason == 0) {
-            if (bestp) {
-                p->pwf.cant_fetch_work_reason = CANT_FETCH_WORK_NOT_HIGHEST_PRIORITY;
-                if (log_flags.work_fetch_debug) {
-                    msg_printf(0, MSG_INFO,
-                        "[work_fetch] not piggybacking work req: %s has higher priority",
-                        bestp->get_project_name()
-                    );
-                }
-            } else {
-                p->pwf.cant_fetch_work_reason = CANT_FETCH_WORK_DONT_NEED;
-            }
+
+    // if project was updated from manager and config says so,
+    // fetch work for a resource even if there are higher-prio projects
+    // able to fetch it
+    //
+    bool check_higher_priority_projects = true;
+    if (p->sched_rpc_pending && config.fetch_on_update) {
+        check_higher_priority_projects = false;
+    }
+
+    // For each resource, scan projects in decreasing priority,
+    // seeing if there's one that's higher-priority than this
+    // able to fetch work for the resource.
+    // If not, and the resource needs topping off, do so
+    //
+    for (int i=0; i<coprocs.n_rsc; i++) {
+        DEBUG(msg_printf(p, MSG_INFO, "piggyback: resource %s", rsc_name(i));)
+        if (i && !gpus_usable) continue;
+        RSC_WORK_FETCH& rwf = rsc_work_fetch[i];
+        if (!rwf.can_fetch(p)) {
+            DEBUG(msg_printf(p, MSG_INFO, "piggyback: can't fetch %s", rsc_name(i));)
+            continue;
         }
-        clear_request();
+        bool buffer_low = (rwf.saturated_time < gstate.work_buf_total());
+        if (!buffer_low && !rwf.uses_starved_excluded_instances(p)) {
+            DEBUG(msg_printf(p, MSG_INFO, "piggyback: don't need %s", rsc_name(i));)
+            continue;
+        }
+        if (check_higher_priority_projects) {
+            PROJECT* p2 = NULL;
+            for (unsigned int j=0; j<gstate.projects.size(); j++) {
+                p2 = gstate.projects[j];
+                if (p2 == p) break;
+                if (p2->pwf.cant_fetch_work_reason) {
+                    DEBUG(msg_printf(p, MSG_INFO, "piggyback: %s can't fetch work", p2->project_name);)
+                    continue;
+                }
+                if (rwf.can_fetch(p2) && !rwf.backed_off(p2)) {
+                    DEBUG(msg_printf(p, MSG_INFO, "piggyback: better proj %s", p2->project_name);)
+                    break;
+                }
+            }
+            if (p != p2) continue;
+        }
+        DEBUG(msg_printf(p, MSG_INFO, "piggyback: requesting %s", rsc_name(i));)
+        if (buffer_low) {
+            rwf.set_request(p);
+        } else {
+            rwf.set_request_excluded(p);
+        }
+    }
+    if (!requested_work()) {
+        p->pwf.cant_fetch_work_reason = CANT_FETCH_WORK_DONT_NEED;
     }
 }
 
@@ -635,7 +553,7 @@ PROJECT* WORK_FETCH::non_cpu_intensive_project_needing_work() {
         if (!p->non_cpu_intensive) continue;
         if (!p->can_request_work()) continue;
         if (p->rsc_pwf[0].backoff_time > gstate.now) continue;
-        if (has_a_job(p)) continue;
+        if (has_a_job_in_progress(p)) continue;
         clear_request();
         rsc_work_fetch[0].req_secs = 1;
         return p;
@@ -643,29 +561,98 @@ PROJECT* WORK_FETCH::non_cpu_intensive_project_needing_work() {
     return 0;
 }
 
-// choose a project to fetch work from,
-// and set the request fields of resource objects.
-//
-PROJECT* WORK_FETCH::choose_project(
-    bool strict_hyst,
-    PROJECT* backoff_exempt_project
-) {
-    PROJECT* p;
+static bool higher_priority(PROJECT *p1, PROJECT *p2) {
+    return (p1->sched_priority > p2->sched_priority);
+}
 
-    if (log_flags.work_fetch_debug) {
-        msg_printf(0, MSG_INFO, "[work_fetch] work fetch start");
+// check resource-level backoff
+//
+bool RSC_WORK_FETCH::backed_off(PROJECT* p) {
+    if (project_state(p).backoff_time > gstate.now) {
+        DEBUG(msg_printf(p, MSG_INFO, "skip: backoff");)
+        return true;
+    }
+    return false;
+}
+
+// a variety of checks for whether we should ask this project
+// for work of this type
+//
+bool RSC_WORK_FETCH::can_fetch(PROJECT *p) {
+    // see whether work fetch for this resource is banned
+    // by prefs, config, project, or acct mgr
+    //
+    if (dont_fetch(p, rsc_type)) {
+        DEBUG(msg_printf(p, MSG_INFO, "skip: dont_fetch");)
+        return false;
     }
 
-    p = non_cpu_intensive_project_needing_work();
-    if (p) return p;
+    RSC_PROJECT_WORK_FETCH& rpwf = project_state(p);
 
+    // if project has zero resource share,
+    // only fetch work if a device is idle
+    //
+    if (p->resource_share == 0 && nidle_now == 0) {
+        DEBUG(msg_printf(p, MSG_INFO, "skip: zero share");)
+        return false;
+    }
+
+    // if project has excluded GPUs of this type,
+    // we need to avoid fetching work just because there's an idle instance
+    // or a shortfall;
+    // fetching work might not alleviate either of these,
+    // and we'd end up fetching unbounded work.
+    // At the same time, we want to respect work buf params if possible.
+    //
+    // Current policy:
+    // don't fetch work if remaining time of this project's jobs
+    // exceeds work_buf_min * (#usable instances / #instances)
+    //
+    // TODO: THIS IS FAIRLY CRUDE. Making it smarter would require
+    // computing shortfall etc. on a per-project basis
+    //
+    int nexcl = rpwf.ncoprocs_excluded;
+    if (rsc_type && nexcl) {
+        int n_not_excluded = ninstances - nexcl;
+        if (rpwf.n_runnable_jobs >= n_not_excluded
+            && rpwf.queue_est > (gstate.work_buf_min() * n_not_excluded)/ninstances
+        ) {
+            DEBUG(msg_printf(p, MSG_INFO, "skip: too much work");)
+            return false;
+        }
+    }
+
+    if (rpwf.anon_skip) {
+        DEBUG(msg_printf(p, MSG_INFO, "skip: anon");)
+        return false;
+    }
+    return true;
+}
+
+// return true if there is exclusion starvation
+// and this project can use the starved instances
+//
+bool RSC_WORK_FETCH::uses_starved_excluded_instances(PROJECT* p) {
+    RSC_PROJECT_WORK_FETCH& rpwf = project_state(p);
+    if (!sim_excluded_instances) return false;
+    if ((sim_excluded_instances & rpwf.non_excluded_instances) == 0) {
+        DEBUG(msg_printf(p, MSG_INFO, "skip: excl");)
+        return false;
+    }
+    return true;
+}
+
+// setup for choose_project() and piggyback()
+//
+void WORK_FETCH::setup() {
     gstate.compute_nuploading_results();
 
     rr_simulation();
     compute_shares();
     project_priority_init(true);
+    clear_request();
 
-    // Decrement the priority of projects that have a lot of work queued.
+    // Decrement the priority of projects that have work queued.
     // Specifically, subtract
     // (FLOPs queued for P)/(FLOPs of max queue)
     // which will generally be between 0 and 1.
@@ -674,26 +661,146 @@ PROJECT* WORK_FETCH::choose_project(
     double max_queued_flops = gstate.work_buf_total()*total_peak_flops();
     for (unsigned int i=0; i<gstate.results.size(); i++) {
         RESULT* rp = gstate.results[i];
-        p = rp->project;
+        PROJECT* p = rp->project;
         p->sched_priority -= rp->estimated_flops_remaining()/max_queued_flops;
     }
 
-    p = 0;
-    if (gpus_usable) {
-        for (int i=1; i<coprocs.n_rsc; i++) {
-            p = rsc_work_fetch[i].choose_project_hyst(strict_hyst, backoff_exempt_project);
-            if (p) break;
+    // don't request work from projects w/ > 1000 runnable jobs
+    //
+    int job_limit = 1000;
+    for (unsigned int i=0; i<gstate.projects.size(); i++) {
+        PROJECT* p = gstate.projects[i];
+        if (p->pwf.n_runnable_jobs > job_limit && !p->pwf.cant_fetch_work_reason) {
+            p->pwf.cant_fetch_work_reason = CANT_FETCH_WORK_TOO_MANY_RUNNABLE;
         }
     }
-    if (!p) {
-        p = rsc_work_fetch[0].choose_project_hyst(strict_hyst, backoff_exempt_project);
-    }
 
+    std::sort(
+        gstate.projects.begin(),
+        gstate.projects.end(),
+        higher_priority
+    );
     if (log_flags.work_fetch_debug) {
         print_state();
-        if (!p) {
+    }
+}
+
+// Choose a project to fetch work from,
+// and set the request fields of resource objects.
+// Set p->sched_rpc_pending; if you decide not to request work
+// from the project, you must clear this.
+//
+PROJECT* WORK_FETCH::choose_project() {
+    PROJECT* p;
+
+    if (log_flags.work_fetch_debug) {
+        msg_printf(0, MSG_INFO, "[work_fetch] entering choose_project()");
+    }
+
+    p = non_cpu_intensive_project_needing_work();
+    if (p) return p;
+
+    setup();
+
+    for (int i=0; i<coprocs.n_rsc; i++) {
+        rsc_work_fetch[i].found_project = NULL;
+    }
+
+    // scan projects in order of decreasing priority
+    //
+    bool found = false;
+    for (unsigned int j=0; j<gstate.projects.size(); j++) {
+        p = gstate.projects[j];
+        DEBUG(msg_printf(p, MSG_INFO, "scanning");)
+        if (p->pwf.cant_fetch_work_reason) {
+            DEBUG(msg_printf(p, MSG_INFO, "skip: cfwr %d", p->pwf.cant_fetch_work_reason);)
+            continue;
+        }
+
+        // For each resource type:
+        // - See if we can ask this project for work of that type;
+        //   if so set a flag so that lower-priority projects
+        //   won't request it
+        // - If so, see if work is needed for this type;
+        //   if so, set "found_project" flag
+        //
+        int rsc_index = -1;
+        for (int i=0; i<coprocs.n_rsc; i++) {
+            if (i && !gpus_usable) continue;
+            RSC_WORK_FETCH& rwf = rsc_work_fetch[i];
+            if (rwf.can_fetch(p) && !rwf.backed_off(p)) {
+                if (!rwf.found_project) {
+                    rwf.found_project = p;
+                }
+                DEBUG(msg_printf(p, MSG_INFO, "can fetch %s", rsc_name(i));)
+            } else {
+                DEBUG(msg_printf(p, MSG_INFO, "can't fetch %s", rsc_name(i));)
+                continue;
+            }
+            bool buffer_low = (rwf.saturated_time < gstate.work_buf_min());
+            if (buffer_low || rwf.uses_starved_excluded_instances(p)) {
+                DEBUG(msg_printf(p, MSG_INFO, "%s needs work", rsc_name(i));)
+                rsc_index = i;
+                break;
+            }
+        }
+
+        // If rsc_index is nonzero, it's a resource that this project
+        // can ask for work, and which needs work.
+        // And this is the highest-priority project having this property.
+        // Request work from this resource,
+        // and any others for which this is the highest-priority project
+        // able to request work
+        //
+        if (rsc_index >= 0) {
+            bool any_request = false;
+            for (int i=0; i<coprocs.n_rsc; i++) {
+                if (i && !gpus_usable) continue;
+                RSC_WORK_FETCH& rwf = rsc_work_fetch[i];
+                bool buffer_low;
+                DEBUG(msg_printf(p, MSG_INFO, "checking %s", rsc_name(i));)
+                if (i == rsc_index) {
+                    buffer_low = (rwf.saturated_time < gstate.work_buf_min());
+                } else {
+                    if (rwf.found_project && rwf.found_project != p) {
+                        DEBUG(msg_printf(p, MSG_INFO, "%s not high prio proj", rsc_name(i));)
+                        continue;
+                    }
+                    buffer_low = (rwf.saturated_time < gstate.work_buf_total());
+                    if (!buffer_low && !rwf.uses_starved_excluded_instances(p)) {
+                        DEBUG(msg_printf(p, MSG_INFO, "%s don't need", rsc_name(i));)
+                        continue;
+                    }
+                    if (!rwf.can_fetch(p)) {
+                        DEBUG(msg_printf(p, MSG_INFO, "%s can't fetch", rsc_name(i));)
+                        continue;
+                    }
+                }
+                if (buffer_low) {
+                    rwf.set_request(p);
+                    DEBUG(msg_printf(p, MSG_INFO, "%s set_request: %f", rsc_name(i), rwf.req_secs);)
+                } else {
+                    rwf.set_request_excluded(p);
+                    DEBUG(msg_printf(p, MSG_INFO, "%s set_request_excluded: %f", rsc_name(i), rwf.req_secs);)
+                }
+                if (rwf.req_secs > 0) {
+                    any_request = true;
+                }
+            }
+            if (any_request) {
+                found = true;
+                break;
+            }
+        }
+    }
+
+    if (found) {
+        p->sched_rpc_pending = RPC_REASON_NEED_WORK;
+    } else {
+        if (log_flags.work_fetch_debug) {
             msg_printf(0, MSG_INFO, "[work_fetch] No project chosen for work fetch");
         }
+        p = NULL;
     }
 
     return p;
@@ -784,7 +891,7 @@ void WORK_FETCH::write_request(FILE* f, PROJECT* p) {
     if (log_flags.work_fetch_debug) {
         char buf[256];
         request_string(buf);
-        msg_printf(p, MSG_INFO, buf);
+        msg_printf(p, MSG_INFO, "%s", buf);
     }
 }
 
@@ -793,34 +900,41 @@ void WORK_FETCH::write_request(FILE* f, PROJECT* p) {
 void WORK_FETCH::handle_reply(
     PROJECT* p, SCHEDULER_REPLY*, vector<RESULT*> new_results
 ) {
-    bool got_rsc[MAX_RSC];
+    bool got_work[MAX_RSC];
+    bool requested_work_rsc[MAX_RSC];
     for (int i=0; i<coprocs.n_rsc; i++) {
-        got_rsc[i] = false;
+        got_work[i] = false;
+        requested_work_rsc[i] = (rsc_work_fetch[i].req_secs > 0);
+    }
+    for (unsigned int i=0; i<new_results.size(); i++) {
+        RESULT* rp = new_results[i];
+        got_work[rp->avp->gpu_usage.rsc_type] = true;
     }
 
-    // if didn't get any jobs, back off on requested resource types
-    //
-    if (!new_results.size()) {
-        // but not if RPC was requested by project
+    for (int i=0; i<coprocs.n_rsc; i++) {
+        // back off on a resource type if
+        // - we asked for jobs
+        // - we didn't get any
+        // - we're not currently backed off for that type
+        //   (i.e. don't back off because of a piggyback request)
+        // - the RPC was done for a reason that is automatic
+        //   and potentially frequent
         //
-        if (p->sched_rpc_pending != RPC_REASON_PROJECT_REQ) {
-            for (int i=0; i<coprocs.n_rsc; i++) {
-                if (rsc_work_fetch[i].req_secs) {
+        if (requested_work_rsc[i] && !got_work[i]) {
+            if (p->rsc_pwf[i].backoff_time < gstate.now) {
+                switch (p->sched_rpc_pending) {
+                case RPC_REASON_RESULTS_DUE:
+                case RPC_REASON_NEED_WORK:
+                case RPC_REASON_TRICKLE_UP:
                     p->rsc_pwf[i].resource_backoff(p, rsc_name(i));
                 }
             }
         }
-        return;
-    }
-
-    // if we did get jobs, clear backoff on resource types
-    //
-    for (unsigned int i=0; i<new_results.size(); i++) {
-        RESULT* rp = new_results[i];
-        got_rsc[rp->avp->gpu_usage.rsc_type] = true;
-    }
-    for (int i=0; i<coprocs.n_rsc; i++) {
-        if (got_rsc[i]) p->rsc_pwf[i].clear_backoff();
+        // if we did get jobs, clear backoff
+        //
+        if (got_work[i]) {
+            p->rsc_pwf[i].clear_backoff();
+        }
     }
 }
 
