@@ -19,11 +19,6 @@
 #pragma implementation "AsyncRPC.h"
 #endif
 
-#if !(defined(_WIN32) || (defined(__WXMAC__) && (MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_4)))
-#include <xlocale.h>
-//#include "gui_rpc_client.h"
-#endif
-
 #include "stdwx.h"
 #include "BOINCGUIApp.h"
 #include "MainDocument.h"
@@ -35,6 +30,11 @@
 #include "DlgEventLog.h"
 #include "util.h"
 
+#if !(defined(_WIN32) || (defined(__WXMAC__) && (MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_4)))
+#include <xlocale.h>
+#include "gui_rpc_client.h"
+#endif
+
 extern bool s_bSkipExitConfirmation;
 
 #ifdef __WXMAC__
@@ -1004,6 +1004,8 @@ void CMainDocument::HandleCompletedRPC() {
                 exchangeBuf->global_prefs = arg1->global_prefs;
                 exchangeBuf->version_info = arg1->version_info;
                 exchangeBuf->executing_as_daemon = arg1->executing_as_daemon;
+                exchangeBuf->host_info = arg1->host_info;
+                exchangeBuf->time_stats = arg1->time_stats;
                 exchangeBuf->have_nvidia = arg1->have_nvidia;
                 exchangeBuf->have_ati = arg1->have_ati;
             }
@@ -1064,6 +1066,7 @@ void CMainDocument::HandleCompletedRPC() {
             if (!retval) {
                 CachedNoticeUpdate();  // Call this only when notice buffer is stable
             }
+            m_bWaitingForGetNoticesRPC = false;
             break;
         case RPC_GET_MESSAGES:
             if (current_rpc_request.exchangeBuf && !retval) {
