@@ -62,6 +62,9 @@ IMPLEMENT_DYNAMIC_CLASS(CBOINCGUIApp, wxApp)
 BEGIN_EVENT_TABLE (CBOINCGUIApp, wxApp)
     EVT_ACTIVATE_APP(CBOINCGUIApp::OnActivateApp)
     EVT_RPC_FINISHED(CBOINCGUIApp::OnRPCFinished)
+#if (defined(__WXMSW__) && !wxCHECK_VERSION(2, 9, 4))
+    EVT_END_SESSION(CBOINCGUIApp::OnEndSession)
+#endif
 END_EVENT_TABLE ()
 
 
@@ -322,13 +325,15 @@ bool CBOINCGUIApp::OnInit() {
 #if (defined(__WXMAC__) && defined(_DEBUG))     // TODO: implement this for other platforms
         // GDB can't attach to applications which are running as a different user   
         //  or group, so fix up data with current user and group during debugging
-        if (check_security(g_use_sandbox, true)) {
+        if (check_security(g_use_sandbox, true, NULL, 0)) {
             CreateBOINCUsersAndGroups();
             SetBOINCDataOwnersGroupsAndPermissions();
             SetBOINCAppOwnersGroupsAndPermissions(NULL);
         }
 #endif
-        iErrorCode = check_security(g_use_sandbox, true, path_to_error);
+        iErrorCode = check_security(
+            g_use_sandbox, true, path_to_error, sizeof(path_to_error)
+        );
     }
 
     if (iErrorCode) {
@@ -483,6 +488,24 @@ bool CBOINCGUIApp::OnInit() {
 }
 
 
+#if (defined(__WXMSW__) && !wxCHECK_VERSION(2, 9, 4))
+// Work around a bug in wxWidgets 2.8.x which fails 
+// to call OnExit() when Windows is shut down. This
+// is supposed to be fixed in wxWidgets 2.9.x.
+//
+void CBOINCGUIApp::OnEndSession(wxCloseEvent& ) {
+    s_bSkipExitConfirmation = true;
+
+    CBOINCBaseFrame* pFrame = wxGetApp().GetFrame();
+    wxCommandEvent evt(wxEVT_COMMAND_MENU_SELECTED, wxID_EXIT);
+    // The event loop has already been stopped,
+    // so we must call OnExit directly
+    pFrame->OnExit(evt);
+    OnExit();
+}
+#endif
+
+
 int CBOINCGUIApp::OnExit() {
     // Shutdown the System Idle Detection code
     IdleTrackerDetach();
@@ -493,10 +516,12 @@ int CBOINCGUIApp::OnExit() {
         m_pDocument = NULL;
     }
 
-    m_pConfig->SetPath(wxT("/"));
+    // Save Application State
+    SaveState();
+
     if (m_pSkinManager) {
-        m_pConfig->Write(wxT("Skin"), m_pSkinManager->GetSelectedSkin());
         delete m_pSkinManager;
+        m_pSkinManager = NULL;
     }
 
     if (m_pLocale) {
@@ -509,18 +534,24 @@ int CBOINCGUIApp::OnExit() {
         m_pEventLog = NULL;
     }
 
-
-    // Save Application State
-    m_pConfig->Write(wxT("AutomaticallyShutdownClient"), m_iShutdownCoreClient);
-    m_pConfig->Write(wxT("DisplayShutdownClientDialog"), m_iDisplayExitDialog);
-    m_pConfig->Write(wxT("DisableAutoStart"), m_iBOINCMGRDisableAutoStart);
-
     diagnostics_finish();
 
     return wxApp::OnExit();
 }
 
 
+void CBOINCGUIApp::SaveState() {
+    // Save Application State
+    m_pConfig->SetPath(wxT("/"));
+    if (m_pSkinManager) {
+        m_pConfig->Write(wxT("Skin"), m_pSkinManager->GetSelectedSkin());
+    }
+    m_pConfig->Write(wxT("AutomaticallyShutdownClient"), m_iShutdownCoreClient);
+    m_pConfig->Write(wxT("DisplayShutdownClientDialog"), m_iDisplayExitDialog);
+    m_pConfig->Write(wxT("DisableAutoStart"), m_iBOINCMGRDisableAutoStart);
+}
+
+
 ///
 /// Pass the command line parameters and discriptions to wxWidgets for displaying.
 ///
@@ -1003,8 +1034,14 @@ bool CBOINCGUIApp::SetActiveGUI(int iGUISelection, bool bShowWindow) {
             if (pOldFrame) pOldFrame->Hide();
 
             // Delete the old one if it exists
-            // Note: this has the side effect of hiding the Event Log
             if (pOldFrame) pOldFrame->Destroy();
+
+            if (iGUISelection != m_iGUISelected) {
+                m_iGUISelected = iGUISelection;
+                m_pConfig->SetPath(wxT("/"));
+                m_pConfig->Write(wxT("GUISelection"), iGUISelection);
+                m_pConfig->Flush();
+            }
         }
     }
 
@@ -1031,10 +1068,6 @@ bool CBOINCGUIApp::SetActiveGUI(int iGUISelection, bool bShowWindow) {
 #endif
     }
 
-    m_iGUISelected = iGUISelection;
-    m_pConfig->SetPath(wxT("/"));
-    m_pConfig->Write(wxT("GUISelection"), iGUISelection);
-
     wxLogTrace(wxT("Function Start/End"), wxT("CBOINCGUIApp::SetActiveGUI - Function End"));
     return true;
 }
