@@ -402,7 +402,8 @@ CMainDocument::CMainDocument() : rpc(this) {
 
     m_fProjectTotalResourceShare = 0.0;
 
-    m_iMessageSequenceNumber = 0;
+    m_iLastMessageSequenceNumber = 0;
+    m_iFirstMessageSequenceNumber = -1;
 
     m_iNoticeSequenceNumber = 0;
     m_iLastReadNoticeSequenceNumber = -1;
@@ -446,6 +447,8 @@ CMainDocument::CMainDocument() : rpc(this) {
     
     m_dtLasAsyncRPCDlgTime = wxDateTime((time_t)0);
     m_dtLastFrameViewRefreshRPCTime = wxDateTime((time_t)0);
+    
+    status.max_event_log_lines = 0;
 }
 
 
@@ -797,7 +800,7 @@ int CMainDocument::SetGPURunMode(int iMode, int iTimeout) {
         if (iMode == RUN_MODE_RESTORE) {
             GetCoreClientStatus(ccs, true);
         } else {
-            status.network_mode = iMode;
+            status.gpu_mode = iMode;
         }
     }
     return 0;
@@ -954,9 +957,9 @@ void CMainDocument::RunPeriodicRPCs(int frameRefreshRate) {
     //
     request.clear();
     request.which_rpc = RPC_GET_MESSAGES;
-    // m_iMessageSequenceNumber could change between request and execution
+    // m_iLastMessageSequenceNumber could change between request and execution
     // of RPC, so pass in a pointer rather than its value
-    request.arg1 = &m_iMessageSequenceNumber;
+    request.arg1 = &m_iLastMessageSequenceNumber;
     request.arg2 = &messages;
     static bool _true = true;
     request.arg3 = &_true;
@@ -1642,7 +1645,7 @@ void CMainDocument::KillGraphicsApp(HANDLE pid) {
 void CMainDocument::KillGraphicsApp(int pid) {
     char* argv[6];
     char currentDir[1024];
-    char thePIDbuf[10];
+    char thePIDbuf[20];
     int id, iRetVal;
     
 
@@ -2019,21 +2022,36 @@ bool CMainDocument::LocalizeNoticeText(wxString& strMessage, bool bSanitize, boo
 //
 int CMainDocument::CachedMessageUpdate() {
     static bool in_this_func = false;
+    size_t last_ind;
 
     if (in_this_func) return 0;
     in_this_func = true;
 
     if (IsConnected()) {
         // rpc.get_messages is now called from RunPeriodicRPCs()
-        // retval = rpc.get_messages(m_iMessageSequenceNumber, messages);
+        // retval = rpc.get_messages(m_iLastMessageSequenceNumber, messages);
         if (m_iGet_messages_rpc_result) {
             wxLogTrace(wxT("Function Status"), wxT("CMainDocument::CachedMessageUpdate - Get Messages Failed '%d'"), m_iGet_messages_rpc_result);
             m_pNetworkConnection->SetStateDisconnected();
             goto done;
         }
         if (messages.messages.size() != 0) {
-            size_t last_ind = messages.messages.size()-1;
-            m_iMessageSequenceNumber = messages.messages[last_ind]->seqno;
+            last_ind = messages.messages.size()-1;
+            m_iLastMessageSequenceNumber = messages.messages[last_ind]->seqno;
+
+            // status.max_event_log_lines <= 0 means no limit
+            if ((status.max_event_log_lines > 0) &&
+                    (last_ind >= (unsigned)status.max_event_log_lines)
+            ) {
+                // Remove oldest messages if we have too many
+                while (messages.messages.size() > (unsigned)status.max_event_log_lines) {
+                    delete messages.messages.front();
+                    messages.messages.pop_front();
+                }
+                m_iFirstMessageSequenceNumber = messages.messages[0]->seqno;
+            } else if (m_iFirstMessageSequenceNumber < 0) {
+                m_iFirstMessageSequenceNumber = messages.messages[0]->seqno;
+            }
         }
     }
 
@@ -2075,7 +2093,8 @@ int CMainDocument::GetMessageCount() {
 
 int CMainDocument::ResetMessageState() {
     messages.clear();
-    m_iMessageSequenceNumber = 0;
+    m_iLastMessageSequenceNumber = 0;
+    m_iFirstMessageSequenceNumber = -1;
     return 0;
 }
 
