@@ -321,8 +321,11 @@ public:
 
     int                         ResetMessageState();
 
-    int                         m_iMessageSequenceNumber;
+    int                         m_iFirstMessageSequenceNumber;
+    int                         m_iLastMessageSequenceNumber;
 
+    int                         GetFirstMsgSeqNum() { return m_iFirstMessageSequenceNumber; }
+    int                         GetLastMsgSeqNum() { return m_iLastMessageSequenceNumber; }
 
     //
     // Transfers Tab
