@@ -256,7 +256,7 @@ const char** CViewProjects::GetViewIcon() {
 }
 
 
-const int CViewProjects::GetViewCurrentViewPage() {
+int CViewProjects::GetViewCurrentViewPage() {
     return VW_PROJ;
 }
 
@@ -1187,16 +1187,19 @@ wxInt32 CViewProjects::ConvertWebsiteIndexToLink(wxInt32 iProjectIndex, wxInt32
 wxInt32 CViewProjects::ConvertLinkToWebsiteIndex(const wxString& strLink, wxInt32& iProjectIndex, wxInt32& iWebsiteIndex) {
     wxString strTemplate = strLink;
     wxString strBuffer = wxEmptyString;
+    long lProjectIndex, lWebsiteIndex;
 
     strTemplate.Replace(wxT("web:"), wxEmptyString);
 
     strBuffer = strTemplate;
     strBuffer.Remove(strBuffer.Find(wxT(":")));
-    strBuffer.ToLong((long*) &iProjectIndex);
+    strBuffer.ToLong((long*) &lProjectIndex);
+    iProjectIndex = lProjectIndex;
 
     strBuffer = strTemplate;
     strBuffer = strBuffer.Mid(strBuffer.Find(wxT(":")) + 1);
-    strBuffer.ToLong((long*) &iWebsiteIndex);
+    strBuffer.ToLong((long*) &lWebsiteIndex);
+    iWebsiteIndex = lWebsiteIndex;
 
     return 0;
 }
