@@ -18,6 +18,8 @@
 #ifndef _APP_IPC_
 #define _APP_IPC_
 
+#ifdef __cplusplus
+
 #include <vector>
 #include <string>
 #include <cstdio>
@@ -72,13 +74,11 @@ struct SHARED_MEM {
         // app->core
     MSG_CHANNEL graphics_request;
         // core->app
-        // request a graphics mode:
-        // <mode_hide_graphics/>
-        // ...
-        // <mode_blankscreen/>
+        // not currently used
     MSG_CHANNEL graphics_reply;
         // app->core
-        // same as above
+        // <web_graphics_url>
+        // <remote_desktop_addr>
     MSG_CHANNEL heartbeat;
         // core->app
         // <heartbeat/>         sent every second, even while app is suspended
@@ -118,20 +118,10 @@ struct MSG_QUEUE {
 #define SHM_PREFIX          "shm_"
 #define QUIT_PREFIX         "quit_"
 
-struct GRAPHICS_MSG {
-    int mode;
-    char window_station[256];
-    char desktop[256];
-    char display[256];
-
-    GRAPHICS_MSG();
-};
-
 class APP_CLIENT_SHM {
 public:
     SHARED_MEM *shm;
 
-    int decode_graphics_msg(char*, GRAPHICS_MSG&);
     void reset_msgs();        // resets all messages and clears their flags
 
     APP_CLIENT_SHM();
@@ -167,6 +157,7 @@ struct APP_INIT_DATA {
     char result_name[256];
     char authenticator[256];
     int slot;
+    int client_pid;
     double user_total_credit;
     double user_expavg_credit;
     double host_total_credit;
@@ -196,11 +187,16 @@ struct APP_INIT_DATA {
     char gpu_type[64];
     int gpu_device_num;
     int gpu_opencl_dev_index;
+    double gpu_usage;   // APP_VERSION.gpu_usage.usage
 
     // info for multicore apps: how many cores to use
     //
     double ncpus;
 
+    // client configuration info
+    //
+    bool vbox_window;
+
     // Items below here are for BOINC runtime system,
     // and should not be directly accessed by apps
     //
@@ -244,11 +240,10 @@ int parse_graphics_file(FILE* f, GRAPHICS_INFO* gi);
 // other filenames
 #define PROJECT_DIR "projects"
 
-extern const char* xml_graphics_modes[NGRAPHICS_MSGS];
 extern int boinc_link(const char* phys_name, const char* logical_name);
 extern int boinc_resolve_filename_s(const char*, std::string&);
+extern void url_to_project_dir(char* url, char* dir);
 
-#ifdef __cplusplus
 extern "C" {
 #endif
 
@@ -258,6 +253,4 @@ extern int boinc_resolve_filename(const char*, char*, int len);
 } // extern "C" {
 #endif
 
-extern void url_to_project_dir(char* url, char* dir);
-
 #endif
