@@ -19,6 +19,7 @@
 #define _COMMON_DEFS_
 
 #include "miofile.h"
+#include "parse.h"
 
 // #defines or enums that are shared by more than one BOINC component
 // (e.g. client, server, Manager, etc.)
@@ -51,6 +52,7 @@
 #define HTTP_STATUS_PARTIAL_CONTENT         206
 #define HTTP_STATUS_MOVED_PERM              301
 #define HTTP_STATUS_MOVED_TEMP              302
+#define HTTP_STATUS_CLIENT_ERROR            400
 #define HTTP_STATUS_NOT_FOUND               404
 #define HTTP_STATUS_PROXY_AUTH_REQ          407
 #define HTTP_STATUS_RANGE_REQUEST_ERROR     416
@@ -103,12 +105,27 @@ enum SUSPEND_REASON {
     SUSPEND_REASON_EXCLUSIVE_APP_RUNNING = 512,
     SUSPEND_REASON_CPU_USAGE = 1024,
     SUSPEND_REASON_NETWORK_QUOTA_EXCEEDED = 2048,
-    SUSPEND_REASON_OS = 4096
+    SUSPEND_REASON_OS = 4096,
+    SUSPEND_REASON_WIFI_STATE = 4097,
+    SUSPEND_REASON_BATTERY_CHARGING = 4098,
+    SUSPEND_REASON_BATTERY_OVERHEATED = 4099,
+    SUSPEND_REASON_NO_GUI_KEEPALIVE = 4100
 };
 
-// Values of RESULT::state
+// battery state (currently used only for Android)
+//
+enum BATTERY_STATE {
+    BATTERY_STATE_UNKNOWN=0,
+    BATTERY_STATE_DISCHARGING,
+    BATTERY_STATE_CHARGING,
+    BATTERY_STATE_FULL,
+    BATTERY_STATE_OVERHEATED
+};
+
+// Values of RESULT::state in client.
 // THESE MUST BE IN NUMERICAL ORDER
 // (because of the > comparison in RESULT::computing_done())
+// see html/inc/common_defs.inc
 //
 #define RESULT_NEW                  0
     // New result
@@ -127,6 +144,16 @@ enum SUSPEND_REASON {
 #define RESULT_UPLOAD_FAILED        7
     // some output file permanent failure
 
+// Values of FILE_INFO::status.
+// If the status is neither of these two,
+// it's an error code indicating an unrecoverable error
+// in the transfer of the file,
+// or that the file was too big and was deleted.
+//
+#define FILE_NOT_PRESENT    0
+#define FILE_PRESENT        1
+#define FILE_VERIFY_PENDING 2
+
 // values of ACTIVE_TASK::task_state
 //
 #define PROCESS_UNINITIALIZED   0
@@ -167,6 +194,36 @@ enum SUSPEND_REASON {
 #define RPC_REASON_INIT             6
 #define RPC_REASON_PROJECT_REQ      7
 
+struct TIME_STATS {
+// we maintain an exponentially weighted average of these quantities:
+    double now;
+        // the client's time of day
+    double on_frac;
+        // the fraction of total time this host runs the client
+    double connected_frac;
+        // of the time this host runs the client,
+        // the fraction it is connected to the Internet,
+        // or -1 if not known
+    double cpu_and_network_available_frac;
+        // of the time this host runs the client,
+        // the fraction it is connected to the Internet
+        // AND network usage is allowed (by prefs and user toggle)
+        // AND CPU usage is allowed
+    double active_frac;
+        // of the time this host runs the client,
+        // the fraction it is enabled to use CPU
+        // (as determined by preferences, manual suspend/resume, etc.)
+    double gpu_active_frac;
+        // same, GPU
+    double client_start_time;
+    double previous_uptime;
+        // duration of previous session
+
+    void write(MIOFILE&);
+    int parse(XML_PARSER&);
+    void print();
+};
+
 struct VERSION_INFO {
     int major;
     int minor;
@@ -177,6 +234,29 @@ struct VERSION_INFO {
     bool greater_than(VERSION_INFO&);
 };
 
+// used for Android
+//
+struct DEVICE_STATUS {
+    bool on_ac_power;
+    bool on_usb_power;
+    double battery_charge_pct;
+    int battery_state;      // see above
+    double battery_temperature_celsius;
+    bool wifi_online;
+    bool user_active;
+
+    int parse(XML_PARSER&);
+    DEVICE_STATUS() {
+        on_ac_power = false;
+        on_usb_power = false;
+        battery_charge_pct = 0;
+        battery_state =  BATTERY_STATE_UNKNOWN;
+        battery_temperature_celsius = 0;
+        wifi_online = false;
+        user_active = false;
+    }
+};
+
 #define RUN_MUTEX                   "BoincSingleInstance"
 #define CLIENT_AUTH_FILENAME        "client_auth.xml"
 #define LOCK_FILE_NAME              "lockfile"
@@ -190,5 +270,4 @@ struct VERSION_INFO {
 #define DEFAULT_SS_EXECUTABLE       "boincscr"
 #endif
 
-
 #endif
