@@ -72,6 +72,7 @@
 #include "boinc_fcgi.h"
 #endif
 
+//#include "client_types.h"
 #include "miofile.h"
 #include "error_numbers.h"
 #include "parse.h"
@@ -86,6 +87,8 @@
 
 #define MAX_OPENCL_PLATFORMS 16
 
+// arguments to proc_type_name() and proc_type_name_xml().
+//
 #define PROC_TYPE_CPU        0
 #define PROC_TYPE_NVIDIA_GPU 1
 #define PROC_TYPE_AMD_GPU    2
@@ -107,7 +110,7 @@ enum COPROC_USAGE {
     COPROC_UNUSED,
     COPROC_USED
 };
-    
+
 
 // represents a requirement for a coproc.
 // This is a parsed version of the <coproc> elements in an <app_version>
@@ -129,7 +132,7 @@ struct PCI_INFO {
     int parse(XML_PARSER&);
 };
 
-// there's some duplication between the values in 
+// there's some duplication between the values in
 // the OPENCL_DEVICE_PROP struct and the NVIDIA/ATI structs
 //
 struct OPENCL_DEVICE_PROP {
@@ -154,6 +157,8 @@ struct OPENCL_DEVICE_PROP {
                                         // example: "OpenCL 1.1 beta"
     int opencl_device_version_int;      // same, encoded as e.g. 101
     int get_device_version_int();       // call this to encode
+    int opencl_driver_revision;         // OpenCL runtime revision is available
+    int get_opencl_driver_revision();   // call this to encode
     char opencl_driver_version[32];     // For example: "CLH 1.0"
     int device_num;                     // temp used in scan process
     double peak_flops;                  // temp used in scan process
@@ -161,11 +166,9 @@ struct OPENCL_DEVICE_PROP {
     double opencl_available_ram;        // temp used in scan process
     int opencl_device_index;            // temp used in scan process
 
-#ifndef _USING_FCGI_
-    void write_xml(MIOFILE&);
-#endif
-    int parse(XML_PARSER&);
-    void description(char* buf, const char* type);
+    void write_xml(MIOFILE&, const char* tag, bool temp_file=false);
+    int parse(XML_PARSER&, const char* end_tag);
+    void description(char* buf, int buflen, const char* type);
 };
 
 
@@ -186,7 +189,7 @@ struct COPROC {
     bool specified_in_config;
         // If true, this coproc was listed in cc_config.xml
         // rather than being detected by the client.
-    
+
     // the following are used in both client and server for work-fetch info
     //
     double req_secs;
@@ -217,12 +220,12 @@ struct COPROC {
     bool running_graphics_app[MAX_COPROC_INSTANCES];
         // is this GPU running a graphics app (NVIDIA only)
 #if DEFER_ON_GPU_AVAIL_RAM
-   double available_ram_temp[MAX_COPROC_INSTANCES];
+    double available_ram_temp[MAX_COPROC_INSTANCES];
         // used during job scheduling
 #endif
 
     double last_print_time;
-    
+
     OPENCL_DEVICE_PROP opencl_prop;
 
 #ifndef _USING_FCGI_
@@ -270,12 +273,12 @@ struct COPROC {
         return -1;
     }
     void merge_opencl(
-        std::vector<OPENCL_DEVICE_PROP> &opencls, 
+        std::vector<OPENCL_DEVICE_PROP> &opencls,
         std::vector<int>& ignore_dev
     );
     void find_best_opencls(
         bool use_all,
-        std::vector<OPENCL_DEVICE_PROP> &opencls, 
+        std::vector<OPENCL_DEVICE_PROP> &opencls,
         std::vector<int>& ignore_dev
     );
 };
@@ -297,9 +300,9 @@ struct CUDA_DEVICE_PROP {
     double   memPitch;
     int   maxThreadsPerBlock;
     int   maxThreadsDim[3];
-    int   maxGridSize[3]; 
+    int   maxGridSize[3];
     int   clockRate;
-    double   totalConstMem; 
+    double   totalConstMem;
     int   major;     // compute capability
     int   minor;
     double   textureAlignment;
@@ -321,17 +324,15 @@ struct COPROC_NVIDIA : public COPROC {
     COPROC_NVIDIA(): COPROC() {
         clear();
     }
-    void get(
+    void get(std::vector<std::string>& warnings);
+    void correlate(
         bool use_all,
-        std::vector<std::string>&,
         std::vector<int>& ignore_devs
     );
-    void description(char*);
+    void description(char* buf, int buflen);
     void clear();
     int parse(XML_PARSER&);
-    void get_available_ram();
     void set_peak_flops();
-    bool check_running_graphics_app();
     void fake(int driver_version, double ram, double avail_ram, int count);
 
 };
@@ -350,7 +351,7 @@ struct COPROC_ATI : public COPROC {
         // CAL version (not driver version) encoded as an int
     bool atirt_detected;
     bool amdrt_detected;
-    CALdeviceattribs attribs; 
+    CALdeviceattribs attribs;
     CALdeviceinfo info;
     COPROC_USAGE is_used;               // temp used in scan process
 
@@ -360,15 +361,14 @@ struct COPROC_ATI : public COPROC {
     COPROC_ATI(): COPROC() {
         clear();
     }
-    void get(
+    void get(std::vector<std::string>& warnings);
+    void correlate(
         bool use_all,
-        std::vector<std::string>&,
         std::vector<int>& ignore_devs
     );
-    void description(char*);
+    void description(char* buf, int buflen);
     void clear();
     int parse(XML_PARSER&);
-    void get_available_ram();
     void set_peak_flops();
     void fake(double ram, double avail_ram, int);
 };
@@ -385,9 +385,9 @@ struct COPROC_INTEL : public COPROC {
     COPROC_INTEL(): COPROC() {
         clear();
     }
-    void get(
+    void get(std::vector<std::string>& warnings);
+    void correlate(
         bool use_all,
-        std::vector<std::string>&,
         std::vector<int>& ignore_devs
     );
     void clear();
@@ -401,30 +401,50 @@ typedef std::vector<int> IGNORE_GPU_INSTANCE[NPROC_TYPES];
 struct COPROCS {
     int n_rsc;
     COPROC coprocs[MAX_RSC];
+        // array of processor types on this host.
+        // element 0 always represents the CPU.
+        // The remaining elements, if any, are GPUs or other coprocessors
+
+    // The following contain vendor-specific info about GPUs.
+    // (These GPUs are also represented by elements in the coprocs array)
+    //
     COPROC_NVIDIA nvidia;
     COPROC_ATI ati;
     COPROC_INTEL intel_gpu;
 
     void write_xml(MIOFILE& out, bool scheduler_rpc);
     void get(
-        bool use_all, 
+        bool use_all,
         std::vector<std::string> &descs,
         std::vector<std::string> &warnings,
         IGNORE_GPU_INSTANCE &ignore_gpu_instance
     );
-    void get_opencl(
-        bool use_all, 
-        std::vector<std::string> &warnings,
+    void detect_gpus(std::vector<std::string> &warnings);
+    int launch_child_process_to_detect_gpus();
+    void correlate_gpus(
+        bool use_all,
+        std::vector<std::string> &descs,
         IGNORE_GPU_INSTANCE &ignore_gpu_instance
     );
+    void get_opencl(
+        std::vector<std::string> &warnings
+    );
+    void correlate_opencl(
+        bool use_all,
+        IGNORE_GPU_INSTANCE& ignore_gpu_instance
+    );
     cl_int get_opencl_info(
-        OPENCL_DEVICE_PROP& prop, 
-        cl_uint device_index, 
+        OPENCL_DEVICE_PROP& prop,
+        cl_uint device_index,
         std::vector<std::string>& warnings
     );
     int parse(XML_PARSER&);
+    void set_path_to_client(char *path);
+    int write_coproc_info_file(std::vector<std::string> &warnings);
+    int read_coproc_info_file(std::vector<std::string> &warnings);
+    
 #ifdef __APPLE__
-    void opencl_get_ati_mem_size_from_opengl();
+    void opencl_get_ati_mem_size_from_opengl(std::vector<std::string> &warnings);
 #endif
     void summary_string(char* buf, int len);
 
