@@ -33,6 +33,8 @@
 #include <locale.h>
 #endif
 
+#include <deque>
+
 #include "cc_config.h"
 #include "common_defs.h"
 #include "filesys.h"
@@ -125,6 +127,7 @@ struct PROJECT {
     std::string team_name;
     int hostid;
     std::vector<GUI_URL> gui_urls;
+    std::string project_dir;
     double user_total_credit;
     double user_expavg_credit;
     double host_total_credit;      // as reported by server
@@ -456,7 +459,7 @@ struct FILE_TRANSFERS {
 };
 
 struct MESSAGES {
-    std::vector<MESSAGE*> messages;
+    std::deque<MESSAGE*> messages;
 
     MESSAGES();
     ~MESSAGES();
@@ -600,6 +603,7 @@ struct CC_STATUS {
 	double network_mode_delay;
     bool disallow_attach;
     bool simple_gui_only;
+    int max_event_log_lines;
 
     CC_STATUS();
     ~CC_STATUS();
@@ -683,12 +687,14 @@ struct RPC_CLIENT {
     int file_transfer_op(FILE_TRANSFER&, const char*);
     int result_op(RESULT&, const char*);
     int get_host_info(HOST_INFO&);
+    int set_host_info(HOST_INFO&);
     int quit();
     int acct_mgr_info(ACCT_MGR_INFO&);
     const char* mode_name(int mode);
     int get_statistics(PROJECTS&);
     int network_available();
     int get_project_init_status(PROJECT_INIT_STATUS& pis);
+    int report_device_status(DEVICE_STATUS&);
 
     // the following are asynch operations.
     // Make the first call to start the op,
