@@ -46,6 +46,10 @@
 #include "boinc_win.h"
 #endif
 
+#ifdef _MSC_VER
+#define snprintf _snprintf
+#endif
+
 #ifdef _WIN32
 #include "../version.h"
 #else
@@ -267,7 +271,9 @@ int PROJECT::parse(XML_PARSER& xp) {
     char buf[256];
 
     while (!xp.get_tag()) {
-        if (xp.match_tag("/project")) return 0;
+        if (xp.match_tag("/project")) {
+            return 0;
+        }
         if (xp.parse_str("master_url", master_url, sizeof(master_url))) continue;
         if (xp.parse_double("resource_share", resource_share)) continue;
         if (xp.parse_string("project_name", project_name)) continue;
@@ -279,6 +285,7 @@ int PROJECT::parse(XML_PARSER& xp) {
             xml_unescape(team_name);
             continue;
         }
+        if (xp.parse_string("project_dir", project_dir)) continue;
         if (xp.parse_int("hostid", hostid)) continue;
         if (xp.parse_double("user_total_credit", user_total_credit)) continue;
         if (xp.parse_double("user_expavg_credit", user_expavg_credit)) continue;
@@ -299,9 +306,9 @@ int PROJECT::parse(XML_PARSER& xp) {
         if (xp.parse_double("cuda_backoff_time", rsc_desc_nvidia.backoff_time)) continue;
         if (xp.parse_double("cuda_backoff_interval", rsc_desc_nvidia.backoff_interval)) continue;
         if (xp.parse_double("ati_backoff_time", rsc_desc_ati.backoff_time)) continue;
-        if (xp.parse_double("ati_backoff_interval", rsc_desc_intel_gpu.backoff_interval)) continue;
+        if (xp.parse_double("ati_backoff_interval", rsc_desc_ati.backoff_interval)) continue;
         if (xp.parse_double("intel_gpu_backoff_time", rsc_desc_intel_gpu.backoff_time)) continue;
-        if (xp.parse_double("intel_gpu_backoff_interval", rsc_desc_ati.backoff_interval)) continue;
+        if (xp.parse_double("intel_gpu_backoff_interval", rsc_desc_intel_gpu.backoff_interval)) continue;
         if (xp.parse_double("last_rpc_time", last_rpc_time)) continue;
 
         // deprecated elements
@@ -317,11 +324,11 @@ int PROJECT::parse(XML_PARSER& xp) {
                 if (xp.match_tag("/rsc_backoff_time")) {
                     if (!strcmp(buf, "CPU")) {
                         rsc_desc_cpu.backoff_time = value;
-                    } else if (!strcmp(buf, "NVIDIA")) {
+                    } else if (!strcmp(buf, proc_type_name_xml(PROC_TYPE_NVIDIA_GPU))) {
                         rsc_desc_nvidia.backoff_time = value;
-                    } else if (!strcmp(buf, "ATI")) {
+                    } else if (!strcmp(buf, proc_type_name_xml(PROC_TYPE_AMD_GPU))) {
                         rsc_desc_ati.backoff_time = value;
-                    } else if (!strcmp(buf, "INTEL_GPU")) {
+                    } else if (!strcmp(buf, proc_type_name_xml(PROC_TYPE_INTEL_GPU))) {
                         rsc_desc_intel_gpu.backoff_time = value;
                     }
                     break;
@@ -337,11 +344,11 @@ int PROJECT::parse(XML_PARSER& xp) {
                 if (xp.match_tag("/rsc_backoff_interval")) {
                     if (!strcmp(buf, "CPU")) {
                         rsc_desc_cpu.backoff_interval = value;
-                    } else if (!strcmp(buf, "NVIDIA")) {
+                    } else if (!strcmp(buf, proc_type_name_xml(PROC_TYPE_NVIDIA_GPU))) {
                         rsc_desc_nvidia.backoff_interval = value;
-                    } else if (!strcmp(buf, "ATI")) {
+                    } else if (!strcmp(buf, proc_type_name_xml(PROC_TYPE_AMD_GPU))) {
                         rsc_desc_ati.backoff_interval = value;
-                    } else if (!strcmp(buf, "INTEL_GPU")) {
+                    } else if (!strcmp(buf, proc_type_name_xml(PROC_TYPE_INTEL_GPU))) {
                         rsc_desc_intel_gpu.backoff_interval = value;
                     }
                     break;
@@ -445,6 +452,7 @@ void PROJECT::clear() {
     strcpy(master_url, "");
     resource_share = 0;
     project_name.clear();
+    project_dir.clear();
     user_name.clear();
     team_name.clear();
     user_total_credit = 0;
@@ -1392,6 +1400,7 @@ int CC_STATUS::parse(XML_PARSER& xp) {
 		if (xp.parse_double("network_mode_delay", network_mode_delay)) continue;
         if (xp.parse_bool("disallow_attach", disallow_attach)) continue;
         if (xp.parse_bool("simple_gui_only", simple_gui_only)) continue;
+        if (xp.parse_int("max_event_log_lines", max_event_log_lines)) continue;
     }
     return ERR_XML_PARSE;
 }
@@ -1738,7 +1747,7 @@ int RPC_CLIENT::project_op(PROJECT& project, const char* op) {
     } else {
         return -1;
     }
-    sprintf(buf,
+    snprintf(buf, sizeof(buf),
         "<%s>\n"
         "  <project_url>%s</project_url>\n"
         "</%s>\n",
@@ -1746,12 +1755,10 @@ int RPC_CLIENT::project_op(PROJECT& project, const char* op) {
         project.master_url,
         tag
     );
+    buf[sizeof(buf)-1] = 0;
     retval = rpc.do_rpc(buf);
-    if (!retval) {
-        retval = rpc.parse_reply();
-    }
-
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 int RPC_CLIENT::project_attach_from_file() {
@@ -1766,19 +1773,19 @@ int RPC_CLIENT::project_attach_from_file() {
         "</project_attach>\n"
     );
     retval = rpc.do_rpc(buf);
-    if (!retval) {
-        retval = rpc.parse_reply();
-    }
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
-int RPC_CLIENT::project_attach(const char* url, const char* auth, const char* name) {
+int RPC_CLIENT::project_attach(
+    const char* url, const char* auth, const char* name
+) {
     int retval;
     SET_LOCALE sl;
     char buf[768];
     RPC rpc(this);
 
-    sprintf(buf,
+    snprintf(buf, sizeof(buf),
         "<project_attach>\n"
         "  <project_url>%s</project_url>\n"
         "  <authenticator>%s</authenticator>\n"
@@ -1786,12 +1793,11 @@ int RPC_CLIENT::project_attach(const char* url, const char* auth, const char* na
         "</project_attach>\n",
         url, auth, name
     );
+    buf[sizeof(buf)-1] = 0;
 
     retval = rpc.do_rpc(buf);
-    if (!retval) {
-        retval = rpc.parse_reply();
-    }
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 int RPC_CLIENT::project_attach_poll(PROJECT_ATTACH_REPLY& reply) {
@@ -1832,7 +1838,8 @@ int RPC_CLIENT::set_run_mode(int mode, double duration) {
     );
 
     retval = rpc.do_rpc(buf);
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 int RPC_CLIENT::set_gpu_mode(int mode, double duration) {
@@ -1850,7 +1857,8 @@ int RPC_CLIENT::set_gpu_mode(int mode, double duration) {
     );
 
     retval = rpc.do_rpc(buf);
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 int RPC_CLIENT::set_network_mode(int mode, double duration) {
@@ -1867,7 +1875,8 @@ int RPC_CLIENT::set_network_mode(int mode, double duration) {
         mode_name(mode), duration
     );
     retval = rpc.do_rpc(buf);
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 int RPC_CLIENT::get_screensaver_tasks(int& suspend_reason, RESULTS& t) {
@@ -1900,7 +1909,8 @@ int RPC_CLIENT::run_benchmarks() {
     RPC rpc(this);
 
     retval = rpc.do_rpc("<run_benchmarks/>\n");
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 int RPC_CLIENT::set_proxy_settings(GR_PROXY_INFO& pi) {
@@ -1909,7 +1919,7 @@ int RPC_CLIENT::set_proxy_settings(GR_PROXY_INFO& pi) {
     char buf[1792];
     RPC rpc(this);
 
-    sprintf(buf,
+    snprintf(buf, sizeof(buf),
         "<set_proxy_settings>\n"
         "    <proxy_info>\n"
         "%s%s%s"
@@ -1937,8 +1947,10 @@ int RPC_CLIENT::set_proxy_settings(GR_PROXY_INFO& pi) {
         pi.socks5_user_passwd.c_str(),
 		pi.noproxy_hosts.c_str()
     );
+    buf[sizeof(buf)-1] = 0;
     retval = rpc.do_rpc(buf);
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 int RPC_CLIENT::get_proxy_settings(GR_PROXY_INFO& p) {
@@ -2021,7 +2033,7 @@ int RPC_CLIENT::file_transfer_op(FILE_TRANSFER& ft, const char* op) {
     } else {
         return -1;
     }
-    sprintf(buf,
+    snprintf(buf, sizeof(buf),
         "<%s>\n"
         "   <project_url>%s</project_url>\n"
         "   <filename>%s</filename>\n"
@@ -2031,8 +2043,10 @@ int RPC_CLIENT::file_transfer_op(FILE_TRANSFER& ft, const char* op) {
         ft.name.c_str(),
         tag
     );
+    buf[sizeof(buf)-1] = 0;
     retval = rpc.do_rpc(buf);
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 int RPC_CLIENT::result_op(RESULT& result, const char* op) {
@@ -2054,7 +2068,7 @@ int RPC_CLIENT::result_op(RESULT& result, const char* op) {
         return -1;
     }
 
-    sprintf(buf,
+    snprintf(buf, sizeof(buf),
         "<%s>\n"
         "   <project_url>%s</project_url>\n"
         "   <name>%s</name>\n"
@@ -2064,8 +2078,10 @@ int RPC_CLIENT::result_op(RESULT& result, const char* op) {
         result.name,
         tag
     );
+    buf[sizeof(buf)-1] = 0;
     retval = rpc.do_rpc(buf);
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 int RPC_CLIENT::get_host_info(HOST_INFO& h) {
@@ -2087,27 +2103,42 @@ int RPC_CLIENT::get_host_info(HOST_INFO& h) {
     return ERR_XML_PARSE;
 }
 
+// set HOST_INFO fields that are easier to get in the GUI than in the client.
+// Currently this is just the product name on Android,
+// which uses a Java version of this function
+//
+int RPC_CLIENT::set_host_info(HOST_INFO& h) {
+    SET_LOCALE sl;
+    RPC rpc(this);
+    char buf[1024];
+
+    snprintf(buf, sizeof(buf),
+        "<set_host_info>\n"
+        "    <host_info>\n"
+        "        <product_name>%s</product_name>\n"
+        "    </host_info>\n"
+        "</set_host_info>\n",
+        h.product_name
+    );
+    buf[sizeof(buf)-1] = 0;
+    int retval = rpc.do_rpc(buf);
+    if (retval) return retval;
+    return rpc.parse_reply();
+}
 
 int RPC_CLIENT::quit() {
     int retval;
-    char buf[256];
     SET_LOCALE sl;
     RPC rpc(this);
 
     retval = rpc.do_rpc("<quit/>\n");
     if (retval) return retval;
-    while (rpc.fin.fgets(buf, 256)) {
-        if (match_tag(buf, "success")) {
-            return 0;
-        }
-        if (strstr(buf, "unauthorized")) {
-            return ERR_AUTHENTICATOR;
-        }
-    }
-    return ERR_XML_PARSE;
+    return rpc.parse_reply();
 }
 
-int RPC_CLIENT::acct_mgr_rpc(const char* url, const char* name, const char* password, bool use_config_file) {
+int RPC_CLIENT::acct_mgr_rpc(
+    const char* url, const char* name, const char* password, bool use_config_file
+) {
     int retval;
     SET_LOCALE sl;
     char buf[1024];
@@ -2120,7 +2151,7 @@ int RPC_CLIENT::acct_mgr_rpc(const char* url, const char* name, const char* pass
             "</acct_mgr_rpc>\n"
         );
     } else {
-        sprintf(buf,
+        snprintf(buf, sizeof(buf),
             "<acct_mgr_rpc>\n"
             "  <url>%s</url>\n"
             "  <name>%s</name>\n"
@@ -2128,9 +2159,11 @@ int RPC_CLIENT::acct_mgr_rpc(const char* url, const char* name, const char* pass
             "</acct_mgr_rpc>\n",
             url, name, password
         );
+        buf[sizeof(buf)-1] = 0;
     }
     retval = rpc.do_rpc(buf);
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 int RPC_CLIENT::acct_mgr_rpc_poll(ACCT_MGR_RPC_REPLY& r) {
@@ -2176,19 +2209,17 @@ int RPC_CLIENT::get_project_config(std::string url) {
     SET_LOCALE sl;
     RPC rpc(this);
 
-    sprintf(buf,
+    snprintf(buf, sizeof(buf),
         "<get_project_config>\n"
         "   <url>%s</url>\n"
         "</get_project_config>\n",
         url.c_str()
     );
+    buf[sizeof(buf)-1] = 0;
 
     retval =  rpc.do_rpc(buf);
-    if (!retval) {
-        retval = rpc.parse_reply();
-    }
-
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 int RPC_CLIENT::get_project_config_poll(PROJECT_CONFIG& pc) {
@@ -2215,7 +2246,7 @@ int RPC_CLIENT::lookup_account(ACCOUNT_IN& ai) {
 
     downcase_string(ai.email_addr);
     string passwd_hash = get_passwd_hash(ai.passwd, ai.email_addr);
-    sprintf(buf,
+    snprintf(buf, sizeof(buf),
         "<lookup_account>\n"
         "   <url>%s</url>\n"
         "   <email_addr>%s</email_addr>\n"
@@ -2225,12 +2256,11 @@ int RPC_CLIENT::lookup_account(ACCOUNT_IN& ai) {
         ai.email_addr.c_str(),
         passwd_hash.c_str()
     );
+    buf[sizeof(buf)-1] = 0;
 
     retval =  rpc.do_rpc(buf);
-    if (!retval) {
-        retval = rpc.parse_reply();
-    }
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 int RPC_CLIENT::lookup_account_poll(ACCOUNT_OUT& ao) {
@@ -2253,7 +2283,7 @@ int RPC_CLIENT::create_account(ACCOUNT_IN& ai) {
 
     downcase_string(ai.email_addr);
     string passwd_hash = get_passwd_hash(ai.passwd, ai.email_addr);
-    sprintf(buf,
+    snprintf(buf, sizeof(buf),
         "<create_account>\n"
         "   <url>%s</url>\n"
         "   <email_addr>%s</email_addr>\n"
@@ -2267,12 +2297,11 @@ int RPC_CLIENT::create_account(ACCOUNT_IN& ai) {
         ai.user_name.c_str(),
         ai.team_name.c_str()
     );
+    buf[sizeof(buf)-1] = 0;
 
     retval =  rpc.do_rpc(buf);
-    if (!retval) {
-        retval = rpc.parse_reply();
-    }
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 int RPC_CLIENT::create_account_poll(ACCOUNT_OUT& ao) {
@@ -2307,7 +2336,9 @@ int RPC_CLIENT::get_newer_version(std::string& version, std::string& version_dow
 int RPC_CLIENT::read_global_prefs_override() {
     SET_LOCALE sl;
     RPC rpc(this);
-    return rpc.do_rpc("<read_global_prefs_override/>");
+    int retval = rpc.do_rpc("<read_global_prefs_override/>");
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 int RPC_CLIENT::get_global_prefs_file(string& s) {
@@ -2422,14 +2453,17 @@ int RPC_CLIENT::set_global_prefs_override(string& s) {
     RPC rpc(this);
     char buf[64000];
 
-    sprintf(buf,
+    snprintf(buf, sizeof(buf),
         "<set_global_prefs_override>\n"
         "%s\n"
         "</set_global_prefs_override>\n",
         s.c_str()
     );
+    buf[sizeof(buf)-1] = 0;
+
     retval = rpc.do_rpc(buf);
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 int RPC_CLIENT::get_global_prefs_override_struct(GLOBAL_PREFS& prefs, GLOBAL_PREFS_MASK& mask) {
@@ -2469,7 +2503,8 @@ int RPC_CLIENT::read_cc_config() {
     RPC rpc(this);
 
     retval = rpc.do_rpc("<read_cc_config/>");
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 int RPC_CLIENT::get_cc_config(CONFIG& config, LOG_FLAGS& log_flags) {
@@ -2494,7 +2529,8 @@ int RPC_CLIENT::set_cc_config(CONFIG& config, LOG_FLAGS& log_flags) {
     config.write(mf, log_flags);
 
     retval = rpc.do_rpc(buf);
-    return retval;
+    if (retval) return retval;
+    return rpc.parse_reply();
 }
 
 static int parse_notices(XML_PARSER& xp, NOTICES& notices) {
