@@ -68,11 +68,13 @@ void HOST_INFO::clear_host_info() {
 
     strcpy(os_name, "");
     strcpy(os_version, "");
+    strcpy(product_name, "");
 
     strcpy(virtualbox_version, "");
+    have_cpu_opencl = false;
 }
 
-int HOST_INFO::parse(XML_PARSER& xp, bool benchmarks_only) {
+int HOST_INFO::parse(XML_PARSER& xp, bool static_items_only) {
     while (!xp.get_tag()) {
         if (xp.match_tag("/host_info")) return 0;
         if (xp.parse_double("p_fpops", p_fpops)) {
@@ -91,15 +93,19 @@ int HOST_INFO::parse(XML_PARSER& xp, bool benchmarks_only) {
         }
         if (xp.parse_double("p_calculated", p_calculated)) continue;
         if (xp.parse_bool("p_vm_extensions_disabled", p_vm_extensions_disabled)) continue;
+        if (xp.parse_str("host_cpid", host_cpid, sizeof(host_cpid))) continue;
+#ifdef ANDROID
+        if (xp.parse_str("product_name", product_name, sizeof(product_name))) continue;
+#endif
 
-        if (benchmarks_only) continue;
+        if (static_items_only) continue;
 
-        // WARNING: NOTHING AFTER HERE IS READ FROM STATE FILE ON THE CLIENT
+        // Items after here are determined dynamically at startup,
+        // so don't parse them from the state file
 
         if (xp.parse_int("timezone", timezone)) continue;
         if (xp.parse_str("domain_name", domain_name, sizeof(domain_name))) continue;
         if (xp.parse_str("ip_addr", ip_addr, sizeof(ip_addr))) continue;
-        if (xp.parse_str("host_cpid", host_cpid, sizeof(host_cpid))) continue;
         if (xp.parse_int("p_ncpus", p_ncpus)) continue;
         if (xp.parse_str("p_vendor", p_vendor, sizeof(p_vendor))) continue;
         if (xp.parse_str("p_model", p_model, sizeof(p_model))) continue;
@@ -118,6 +124,10 @@ int HOST_INFO::parse(XML_PARSER& xp, bool benchmarks_only) {
         if (xp.match_tag("coprocs")) {
             this->coprocs.parse(xp);
         }
+        if (xp.match_tag("cpu_opencl_prop")) {
+            int retval = cpu_opencl_prop.parse(xp, "/cpu_opencl_prop");
+            if (!retval) have_cpu_opencl = true;
+        }
     }
     return ERR_XML_PARSE;
 }
@@ -134,7 +144,7 @@ int HOST_INFO::parse(XML_PARSER& xp, bool benchmarks_only) {
 int HOST_INFO::write(
     MIOFILE& out, bool include_net_info, bool include_coprocs
 ) {
-    char pv[265], pm[256], pf[1024], osn[256], osv[256];
+    char pv[265], pm[256], pf[1024], osn[256], osv[256], pn[256];
     out.printf(
         "<host_info>\n"
         "    <timezone>%d</timezone>\n",
@@ -189,6 +199,13 @@ int HOST_INFO::write(
         osn,
         osv
     );
+    if (strlen(product_name)) {
+        xml_escape(product_name, pn, sizeof(pn));
+        out.printf(
+            "    <product_name>%s</product_name>\n",
+            pn
+        );
+    }
     if (strlen(virtualbox_version)) {
         char buf[256];
         xml_escape(virtualbox_version, buf, sizeof(buf));
@@ -200,6 +217,9 @@ int HOST_INFO::write(
     if (include_coprocs) {
         this->coprocs.write_xml(out, false);
     }
+    if (have_cpu_opencl) {
+        cpu_opencl_prop.write_xml(out, "cpu_opencl_prop");
+    }
     out.printf(
         "</host_info>\n"
     );
