@@ -30,6 +30,7 @@
 
 #include "miofile.h"
 #include "coproc.h"
+#include "common_defs.h"
 
 // if you add fields, update clear_host_info()
 
@@ -60,6 +61,8 @@ public:
 
     char os_name[256];
     char os_version[256];
+    char product_name[256];       // manufacturer and/or model of system
+                                  // currently used for Android devices
 
     // the following is non-empty if VBox is installed
     //
@@ -67,15 +70,11 @@ public:
 
     COPROCS coprocs;
 
-#ifdef ANDROID
-    int battery_charge_pct;
-    int battery_state;
-    double battery_temperature_celsius;
-    void get_battery_status();
-#endif
+    bool have_cpu_opencl;
+    OPENCL_DEVICE_PROP cpu_opencl_prop;
 
     HOST_INFO();
-    int parse(XML_PARSER&, bool benchmarks_only = false);
+    int parse(XML_PARSER&, bool static_items_only = false);
     int write(MIOFILE&, bool include_net_info, bool include_coprocs);
     int parse_cpu_benchmarks(FILE*);
     int write_cpu_benchmarks(FILE*);
@@ -86,9 +85,6 @@ public:
     bool users_idle(bool check_all_logins, double idle_time_to_run, double *actual_idle_time=NULL);
 #else
     bool users_idle(bool check_all_logins, double idle_time_to_run);
-#endif
-#ifdef ANDROID
-    bool host_wifi_online();
 #endif
     int get_host_info();
     int get_host_battery_charge();
