@@ -46,6 +46,8 @@ void GLOBAL_PREFS_MASK::clear() {
 }
 
 void GLOBAL_PREFS_MASK::set_all() {
+    battery_charge_min_pct = true;
+    battery_max_temperature = true;
     confirm_before_connecting = true;
     cpu_scheduling_period_minutes = true;
     cpu_usage_limit = true;
@@ -81,6 +83,8 @@ void GLOBAL_PREFS_MASK::set_all() {
 }
 
 bool GLOBAL_PREFS_MASK::are_prefs_set() {
+    if (battery_charge_min_pct) return true;
+    if (battery_max_temperature) return true;
     if (confirm_before_connecting) return true;
     if (cpu_scheduling_period_minutes) return true;
     if (cpu_usage_limit) return true;
@@ -205,6 +209,8 @@ void WEEK_PREFS::unset(int day) {
 // so that the client can do the RPC and get the global prefs from the server
 //
 void GLOBAL_PREFS::defaults() {
+    battery_charge_min_pct = 95;
+    battery_max_temperature = 45;
     confirm_before_connecting = true;
     cpu_scheduling_period_minutes = 60;
     cpu_times.clear();
@@ -386,6 +392,14 @@ int GLOBAL_PREFS::parse_override(
             }
             continue;
         }
+        if (xp.parse_double("battery_charge_min_pct", battery_charge_min_pct)) {
+            mask.battery_charge_min_pct = true;
+            continue;
+        }
+        if (xp.parse_double("battery_max_temperature", battery_max_temperature)) {
+            mask.battery_max_temperature = true;
+            continue;
+        }
         if (xp.parse_bool("run_on_batteries", run_on_batteries)) {
             mask.run_on_batteries = true;
             continue;
@@ -584,6 +598,8 @@ int GLOBAL_PREFS::write(MIOFILE& f) {
         "<global_preferences>\n"
         "   <source_project>%s</source_project>\n"
         "   <mod_time>%f</mod_time>\n"
+        "   <battery_charge_min_pct>%f</battery_charge_min_pct>\n"
+        "   <battery_max_temperature>%f</battery_max_temperature>\n"
         "   <run_on_batteries>%d</run_on_batteries>\n"
         "   <run_if_user_active>%d</run_if_user_active>\n"
         "   <run_gpu_if_user_active>%d</run_gpu_if_user_active>\n"
@@ -618,6 +634,8 @@ int GLOBAL_PREFS::write(MIOFILE& f) {
         "   <network_wifi_only>%d</network_wifi_only>\n",
         source_project,
         mod_time,
+        battery_charge_min_pct,
+        battery_max_temperature,
         run_on_batteries?1:0,
         run_if_user_active?1:0,
         run_gpu_if_user_active?1:0,
@@ -746,6 +764,16 @@ int GLOBAL_PREFS::write_subset(MIOFILE& f, GLOBAL_PREFS_MASK& mask) {
             leave_apps_in_memory?1:0
         );
     }
+    if (mask.battery_charge_min_pct) {
+        f.printf("   <battery_charge_min_pct>%f</battery_charge_min_pct>\n",
+            battery_charge_min_pct
+        );
+    }
+    if (mask.battery_max_temperature) {
+        f.printf("   <battery_max_temperature>%f</battery_max_temperature>\n",
+            battery_max_temperature
+        );
+    }
     if (mask.confirm_before_connecting) {
         f.printf("   <confirm_before_connecting>%d</confirm_before_connecting>\n",
             confirm_before_connecting?1:0
